"""
Framework App Config
"""

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class FrameworkConfig(AppConfig):
    """
    Framework App Config
    """

    name = "allianceauth.framework"
    label = "framework"
    verbose_name = _("Framework")



"""API for interacting with celery workers."""

import itertools
import logging
from typing import Optional

from amqp.exceptions import ChannelError
from celery import current_app

from django.conf import settings

logger = logging.getLogger(__name__)


def active_tasks_count() -> Optional[int]:
    """Return count of currently active tasks
    or None if celery workers are not online.
    """
    inspect = current_app.control.inspect()
    return _tasks_count(inspect.active())


def _tasks_count(data: dict) -> Optional[int]:
    """Return count of tasks in data from celery inspect API."""
    try:
        tasks = itertools.chain(*data.values())
    except AttributeError:
        return None
    return len(list(tasks))


def queued_tasks_count() -> Optional[int]:
    """Return count of queued tasks. Return None if there was an error."""
    try:
        with current_app.connection_or_acquire() as conn:
            result = conn.default_channel.queue_declare(
                queue=getattr(settings, "CELERY_DEFAULT_QUEUE", "celery"), passive=True
            )
            return result.message_count

    except ChannelError:
        # Queue doesn't exist, probably empty
        return 0

    except Exception:
        logger.exception("Failed to get celery queue length")

    return None



from typing import List, NamedTuple, Optional
from unittest.mock import patch

from bs4 import BeautifulSoup

from django.test import RequestFactory, TestCase

from allianceauth.menu.templatetags.menu_menu_items import (
    RenderedMenuItem,
    render_menu,
)
from allianceauth.menu.tests.factories import (
    create_app_menu_item,
    create_folder_menu_item,
    create_link_menu_item,
    create_menu_item_from_hook,
    create_menu_item_hook_class,
    create_menu_item_hook_function,
    create_rendered_menu_item,
)
from allianceauth.menu.tests.utils import (
    PACKAGE_PATH,
    remove_whitespaces,
    render_template,
)

MODULE_PATH = PACKAGE_PATH + ".templatetags.menu_menu_items"


class TestTemplateTags(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.factory = RequestFactory()

    @patch(MODULE_PATH + ".render_menu", spec=True)
    @patch(MODULE_PATH + ".smart_sync.sync_menu", spec=True)
    def test_sorted_menu_items(self, mock_sync_menu, mock_render_menu):
        # given
        fake_item = {"html": "Alpha"}
        mock_render_menu.return_value = [fake_item]
        request = self.factory.get("/")

        # when
        rendered = render_template(
            "{% load menu_menu_items %}{% menu_items %}",
            context={"request": request},
        )
        self.assertIn("Alpha", rendered)
        self.assertTrue(mock_sync_menu.called)


@patch(MODULE_PATH + ".get_hooks", spec=True)
class TestRenderDefaultMenu(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.factory = RequestFactory()

    def test_should_render_app_menu_items(self, mock_get_hooks):
        # given
        menu = [
            create_menu_item_hook_function(text="Charlie", count=42),
            create_menu_item_hook_function(text="Alpha", order=1),
            create_menu_item_hook_function(text="Bravo", order=2),
        ]
        mock_get_hooks.return_value = menu
        for hook in menu:
            create_menu_item_from_hook(hook)

        request = self.factory.get("/")

        # when
        result = render_menu(request)

        # then
        menu = list(result)
        self.assertEqual(len(menu), 3)
        self.assertEqual(menu[0].menu_item.text, "Alpha")
        self.assertEqual(menu[1].menu_item.text, "Bravo")
        self.assertEqual(menu[2].menu_item.text, "Charlie")
        self.assertEqual(menu[2].count, 42)
        attrs = parse_html(menu[2])
        self.assertEqual(attrs.count, 42)
        self.assertEqual(attrs.text, "Charlie")

    def test_should_render_link_menu_items(self, mock_get_hooks):
        # given
        mock_get_hooks.return_value = []
        create_link_menu_item(text="Charlie"),
        create_link_menu_item(text="Alpha", order=1),
        create_link_menu_item(text="Bravo", order=2),

        request = self.factory.get("/")

        # when
        result = render_menu(request)

        # then
        menu = list(result)
        self.assertEqual(len(menu), 3)
        self.assertEqual(menu[0].menu_item.text, "Alpha")
        self.assertEqual(menu[1].menu_item.text, "Bravo")
        self.assertEqual(menu[2].menu_item.text, "Charlie")
        attrs = parse_html(menu[2])
        self.assertEqual(attrs.text, "Charlie")

    def test_should_render_folders(self, mock_get_hooks):
        # given
        mock_get_hooks.return_value = []
        folder = create_folder_menu_item(text="Folder", order=2)
        create_link_menu_item(text="Alpha", order=1)
        create_link_menu_item(text="Bravo", order=3)
        create_link_menu_item(text="Charlie", parent=folder)

        request = self.factory.get("/")

        # when
        result = render_menu(request)

        # then
        menu = list(result)
        self.assertEqual(len(menu), 3)
        self.assertEqual(menu[0].menu_item.text, "Alpha")
        self.assertEqual(menu[1].menu_item.text, "Folder")
        self.assertEqual(menu[2].menu_item.text, "Bravo")

        self.assertEqual(menu[1].children[0].menu_item.text, "Charlie")
        attrs = parse_html(menu[1].children[0])
        self.assertEqual(attrs.text, "Charlie")

    def test_should_render_folder_properties(self, mock_get_hooks):
        # given
        # given
        menu = [
            create_menu_item_hook_function(text="Charlie", count=42),
            create_menu_item_hook_function(text="Alpha", count=5),
            create_menu_item_hook_function(text="Bravo"),
        ]
        mock_get_hooks.return_value = menu

        folder = create_folder_menu_item(text="Folder", order=1)
        for hook in menu:
            create_menu_item_from_hook(hook, parent=folder)

        request = self.factory.get("/")

        # when
        result = render_menu(request)

        # then
        menu = list(result)
        self.assertEqual(len(menu), 1)
        item = menu[0]
        self.assertEqual(item.menu_item.text, "Folder")
        self.assertEqual(item.count, 47)
        self.assertTrue(item.is_folder)
        self.assertEqual(len(item.children), 3)
        attrs = parse_html(item)
        self.assertEqual(attrs.count, 47)
        self.assertIn("fa-folder", attrs.classes)

    def test_should_remove_empty_folders(self, mock_get_hooks):
        # given
        mock_get_hooks.return_value = []
        create_folder_menu_item(text="Folder", order=2)
        create_link_menu_item(text="Alpha", order=1)
        create_link_menu_item(text="Bravo", order=3)

        request = self.factory.get("/")

        # when
        result = render_menu(request)

        # then
        menu = list(result)
        self.assertEqual(len(menu), 2)
        self.assertEqual(menu[0].menu_item.text, "Alpha")
        self.assertEqual(menu[1].menu_item.text, "Bravo")

    def test_should_remove_empty_folders_with_items_hidden(self, mock_get_hooks):
        # given

        class TestHook(create_menu_item_hook_class()):
            text = "Dummy App No Data"
            classes = "fa-solid fa-users-gear"
            url_name = "groupmanagement:management"

            def render(Self, request):
                # simulate no perms
                return ""

        params = {
            "text": "Alpha",
            "classes": "fa-solid fa-users-gear",
            "url_name": "groupmanagement:management",
        }

        alpha = TestHook(**params)

        hooks = [lambda: alpha]

        mock_get_hooks.return_value = hooks

        folder = create_folder_menu_item(text="Folder", order=2)
        create_menu_item_from_hook(hooks[0], parent=folder)
        create_link_menu_item(text="Bravo", order=3)  # this is all that should show

        request = self.factory.get("/")

        # when
        result = render_menu(request)

        # then
        menu = list(result)
        self.assertEqual(len(menu), 1)
        self.assertEqual(menu[0].menu_item.text, "Bravo")

    def test_should_not_include_hidden_items(self, mock_get_hooks):
        # given
        mock_get_hooks.return_value = []
        create_link_menu_item(text="Charlie"),
        create_link_menu_item(text="Alpha", order=1),
        create_link_menu_item(text="Bravo", order=2, is_hidden=True),

        request = self.factory.get("/")

        # when
        result = render_menu(request)

        # then
        menu = list(result)
        self.assertEqual(len(menu), 2)
        self.assertEqual(menu[0].menu_item.text, "Alpha")
        self.assertEqual(menu[1].menu_item.text, "Charlie")

    def test_should_not_render_hidden_folders(self, mock_get_hooks):
        # given
        menu = [
            create_menu_item_hook_function(text="Charlie", count=42),
            create_menu_item_hook_function(text="Alpha", count=5),
            create_menu_item_hook_function(text="Bravo"),
        ]
        mock_get_hooks.return_value = menu

        folder = create_folder_menu_item(text="Folder", order=1, is_hidden=True)
        for hook in menu:
            create_menu_item_from_hook(hook, parent=folder)

        request = self.factory.get("/")

        # when
        result = render_menu(request)

        # then
        menu = list(result)
        self.assertEqual(len(menu), 0)

    def test_should_allow_several_items_with_same_text(self, mock_get_hooks):
        # given
        mock_get_hooks.return_value = []
        create_link_menu_item(text="Alpha", order=1),
        create_link_menu_item(text="Alpha", order=2),

        request = self.factory.get("/")

        # when
        result = render_menu(request)

        # then
        menu = list(result)
        self.assertEqual(len(menu), 2)
        self.assertEqual(menu[0].menu_item.text, "Alpha")
        self.assertEqual(menu[1].menu_item.text, "Alpha")


class TestRenderedMenuItem(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.factory = RequestFactory()
        cls.template = "menu/menu-item-bs5.html"

    def test_create_from_menu_item_with_defaults(self):
        # given
        item = create_link_menu_item()

        # when
        obj = RenderedMenuItem(menu_item=item)

        # then
        self.assertEqual(obj.menu_item, item)
        self.assertIsNone(obj.count)
        self.assertEqual(obj.html, "")
        self.assertEqual(obj.html_id, "")
        self.assertListEqual(obj.children, [])

    def test_should_identify_if_item_is_a_folder(self):
        # given
        app_item = create_rendered_menu_item(menu_item=create_app_menu_item())
        link_item = create_rendered_menu_item(menu_item=create_link_menu_item())
        folder_item = create_rendered_menu_item(menu_item=create_folder_menu_item())

        cases = [
            (app_item, False),
            (link_item, False),
            (folder_item, True),
        ]
        # when
        for obj, expected in cases:
            with self.subTest(type=expected):
                self.assertIs(obj.is_folder, expected)

    def test_should_update_html_for_link_item(self):
        # given
        obj = create_rendered_menu_item(menu_item=create_link_menu_item(text="Alpha"))
        request = self.factory.get("/")

        # when
        obj.update_html(request, self.template)

        # then
        parsed = parse_html(obj)
        self.assertEqual(parsed.text, "Alpha")
        self.assertIsNone(parsed.count)
        self.assertFalse(obj.html_id)

    def test_should_update_html_for_folder_item(self):
        # given
        request = self.factory.get("/")
        folder_item = create_folder_menu_item(text="Alpha")
        link_item = create_link_menu_item(text="Bravo", parent=folder_item)
        obj = create_rendered_menu_item(menu_item=folder_item, count=42)
        rendered_link = create_rendered_menu_item(menu_item=link_item)
        rendered_link.update_html(request, self.template)
        obj.children.append(rendered_link)

        # when
        obj.update_html(request, self.template)

        # then
        self.assertTrue(obj.html_id)
        parsed_parent = parse_html(obj)
        self.assertEqual(parsed_parent.text, "Alpha")
        self.assertEqual(parsed_parent.count, 42)
        self.assertIn("Bravo", obj.html)


class _ParsedMenuItem(NamedTuple):
    classes: List[str]
    text: str
    count: Optional[int]


def parse_html(obj: RenderedMenuItem) -> _ParsedMenuItem:
    soup = BeautifulSoup(obj.html, "html.parser")
    classes = soup.li.i.attrs["class"]
    text = remove_whitespaces(soup.li.a.text)
    try:
        count = int(remove_whitespaces(soup.li.span.text))
    except (AttributeError, ValueError):
        count = None

    return _ParsedMenuItem(classes=classes, text=text, count=count)



"""
Alliance Auth Evecharacter API
"""

from typing import Optional

from django.contrib.auth.models import User

from allianceauth.authentication.models import CharacterOwnership
from allianceauth.eveonline.models import EveCharacter
from allianceauth.framework.api.user import get_sentinel_user


def get_main_character_from_evecharacter(
    character: EveCharacter,
) -> Optional[EveCharacter]:
    """
    Get the main character for a given EveCharacter or None when no main character is set

    :param character:
    :type character:
    :return:
    :rtype:
    """

    try:
        userprofile = character.character_ownership.user.profile
    except (
        AttributeError,
        EveCharacter.character_ownership.RelatedObjectDoesNotExist,
        CharacterOwnership.user.RelatedObjectDoesNotExist,
    ):
        return None

    return userprofile.main_character


def get_user_from_evecharacter(character: EveCharacter) -> User:
    """
    Get the user for an EveCharacter or the sentinel user when no user is found

    :param character:
    :type character:
    :return:
    :rtype:
    """

    try:
        userprofile = character.character_ownership.user.profile
    except (
        AttributeError,
        EveCharacter.character_ownership.RelatedObjectDoesNotExist,
        CharacterOwnership.user.RelatedObjectDoesNotExist,
    ):
        return get_sentinel_user()

    return userprofile.user



from allianceauth.menu.hooks import MenuItemHook
from allianceauth.services.hooks import UrlHook
from django.utils.translation import gettext_lazy as _
from allianceauth import hooks
from allianceauth.corputils import urls


class CorpStats(MenuItemHook):
    def __init__(self):
        MenuItemHook.__init__(
            self,
            _('Corporation Stats'),
            'fa-solid fa-share-nodes',
            'corputils:view',
            navactive=['corputils:']
        )

    def render(self, request):
        if request.user.has_perm('corputils.view_corp_corpstats') or request.user.has_perm(
                'corputils.view_alliance_corpstats') or request.user.has_perm(
                'corputils.add_corpstats') or request.user.has_perm('corputils.view_state_corpstats'):
            return MenuItemHook.render(self, request)
        return ''


@hooks.register('menu_item_hook')
def register_menu():
    return CorpStats()


@hooks.register('url_hook')
def register_url():
    return UrlHook(urls, 'corputils', r'^corpstats/')



from django.utils.translation import gettext_lazy as _

# Overide ESI messages in the dashboard widget
# when the returned messages are not helpful or out of date
ESI_ERROR_MESSAGE_OVERRIDES = {
    420: _("This software has exceeded the error limit for ESI. "
            "If you are a user, please contact the maintainer of this software."
            " If you are a developer/maintainer, please make a greater "
            "effort in the future to receive valid responses. For tips on how, "
            "come have a chat with us in ##3rd-party-dev-and-esi on the EVE "
            "Online Discord. https://www.eveonline.com/discord")
}



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class FatConfig(AppConfig):
    name = 'allianceauth.fleetactivitytracking'
    label = 'fleetactivitytracking'
    verbose_name = _('Fleet Activity Tracking')



import logging

import requests
from packaging.version import InvalidVersion, Version as Pep440Version

from django import template
from django.conf import settings
from django.core.cache import cache

from allianceauth import __version__
from allianceauth.authentication.task_statistics.counters import (
    dashboard_results,
)

register = template.Library()

# cache timers
TAG_CACHE_TIME = 3600  # 1 hours
NOTIFICATION_CACHE_TIME = 300  # 5 minutes
# timeout for all requests
REQUESTS_TIMEOUT = 5    # 5 seconds
# max pages to be fetched from gitlab
MAX_PAGES = 50

GITLAB_AUTH_REPOSITORY_TAGS_URL = (
    'https://gitlab.com/api/v4/projects/allianceauth%2Fallianceauth/repository/tags'
)
GITLAB_AUTH_ANNOUNCEMENT_ISSUES_URL = (
    'https://gitlab.com/api/v4/projects/allianceauth%2Fallianceauth/issues'
    '?labels=announcement&state=opened'
)

logger = logging.getLogger(__name__)


@register.simple_tag()
def decimal_widthratio(this_value, max_value, max_width) -> str:
    if max_value == 0:
        return str(0)

    return str(round(this_value / max_value * max_width, 2))


@register.inclusion_tag('allianceauth/admin-status/overview.html')
def status_overview() -> dict:
    response = {
        "notifications": list(),
        "current_version": __version__,
        "tasks_succeeded": 0,
        "tasks_retried": 0,
        "tasks_failed": 0,
        "tasks_total": 0,
        "tasks_hours": 0,
        "earliest_task": None,
        "debug": settings.DEBUG if settings.DISPLAY_DEBUG else False,
    }
    response.update(_current_notifications())
    response.update(_current_version_summary())
    response.update(_celery_stats())
    return response


def _celery_stats() -> dict:
    hours = getattr(settings, "ALLIANCEAUTH_DASHBOARD_TASKS_MAX_HOURS", 24)
    results = dashboard_results(hours=hours)
    return {
        "tasks_succeeded": results.succeeded,
        "tasks_retried": results.retried,
        "tasks_failed": results.failed,
        "tasks_total": results.total,
        "tasks_hours": results.hours,
        "earliest_task": results.earliest_task,
    }


def _current_notifications() -> dict:
    """returns the newest 5 announcement issues"""
    try:
        notifications = cache.get_or_set(
            'gitlab_notification_issues',
            _fetch_notification_issues_from_gitlab,
            NOTIFICATION_CACHE_TIME
        )
    except requests.HTTPError:
        logger.warning('Error while getting gitlab notifications', exc_info=True)
        top_notifications = []
    else:
        if notifications:
            top_notifications = notifications[:5]
        else:
            top_notifications = []

    response = {
        'notifications': top_notifications,
    }
    return response


def _fetch_notification_issues_from_gitlab() -> list:
    return _fetch_list_from_gitlab(GITLAB_AUTH_ANNOUNCEMENT_ISSUES_URL, max_pages=10)


def _current_version_summary() -> dict:
    """returns the current version info"""
    try:
        tags = cache.get_or_set(
            'git_release_tags', _fetch_tags_from_gitlab, TAG_CACHE_TIME
        )
    except requests.HTTPError:
        logger.warning('Error while getting gitlab release tags', exc_info=True)
        return {}

    if not tags:
        return {}

    (
        latest_patch_version,
        latest_beta_version
    ) = _latests_versions(tags)
    current_version = Pep440Version(__version__)

    has_latest_patch = \
        current_version >= latest_patch_version if latest_patch_version else False
    has_current_beta = \
        current_version <= latest_beta_version \
        and latest_patch_version <= latest_beta_version \
        if latest_beta_version else False

    response = {
        'latest_patch': has_latest_patch,
        'latest_beta': has_current_beta,
        'current_version': str(current_version),
        'latest_patch_version': str(latest_patch_version),
        'latest_beta_version': str(latest_beta_version)
    }
    return response


def _fetch_tags_from_gitlab():
    return _fetch_list_from_gitlab(GITLAB_AUTH_REPOSITORY_TAGS_URL)


def _latests_versions(tags: list) -> tuple:
    """returns latests version from given tags list

    Non-compliant tags will be ignored
    """
    versions = list()
    betas = list()
    for tag in tags:
        try:
            version = Pep440Version(tag.get('name'))
        except InvalidVersion:
            pass
        else:
            if version.is_prerelease or version.is_devrelease:
                betas.append(version)
            else:
                versions.append(version)

    latest_patch_version = max(versions)
    latest_beta_version = max(betas)
    return (
        latest_patch_version,
        latest_beta_version
    )


def _fetch_list_from_gitlab(url: str, max_pages: int = MAX_PAGES) -> list:
    """returns a list from the GitLab API. Supports paging"""
    result = list()

    for page in range(1, max_pages + 1):
        try:
            request = requests.get(
                url, params={'page': page}, timeout=REQUESTS_TIMEOUT
            )
            request.raise_for_status()
        except requests.exceptions.RequestException as e:
            error_str = str(e)

            logger.warning(
                f'Unable to fetch from GitLab API. Error: {error_str}',
                exc_info=True,
            )

            return result

        result += request.json()

        if 'x-total-pages' in request.headers:
            try:
                total_pages = int(request.headers['x-total-pages'])
            except ValueError:
                total_pages = None
        else:
            total_pages = None

        if not total_pages or page >= total_pages:
            break

    return result



from typing import List, Iterable, Callable

from django.urls import include
import esi.urls
from django.conf import settings
from django.contrib import admin
from django.urls import URLPattern, include, path
from django.views.generic.base import TemplateView

import allianceauth.authentication.urls
import allianceauth.authentication.views
import allianceauth.groupmanagement.urls
import allianceauth.notifications.urls
import allianceauth.services.urls
from allianceauth import NAME, views
from allianceauth.authentication import hmac_urls
from allianceauth.authentication.decorators import (
    decorate_url_patterns,
    main_character_required
)
from allianceauth.hooks import get_hooks

admin.site.site_header = NAME


def urls_from_apps(
    apps_hook_functions: Iterable[Callable], public_views_allowed: List[str]
) -> List[URLPattern]:
    """Return urls from apps and add default decorators."""

    url_patterns = []
    allowed_apps = set(public_views_allowed)
    for app_hook_function in apps_hook_functions:
        url_hook = app_hook_function()
        app_pattern = url_hook.include_pattern
        excluded_views = (
            url_hook.excluded_views
            if app_pattern.app_name in allowed_apps
            else None
        )
        url_patterns += [
            path(
                "",
                decorate_url_patterns(
                    [app_pattern], main_character_required, excluded_views
                )
            )
        ]

    return url_patterns


# Functional/Untranslated URL's
urlpatterns = [
    # Locale
    path('i18n/', include('django.conf.urls.i18n')),

    # Authentication
    path('', include(allianceauth.authentication.urls)),
    path('account/login/', TemplateView.as_view(template_name='public/login.html'), name='auth_login_user'),
    path('account/', include(hmac_urls)),

    # Admin urls
    path('admin/', admin.site.urls),

    # SSO
    path('sso/', include((esi.urls, 'esi'), namespace='esi')),
    path('sso/login', allianceauth.authentication.views.sso_login, name='auth_sso_login'),

    # Notifications
    path('', include(allianceauth.notifications.urls)),

    # Groups
    path('', include(allianceauth.groupmanagement.urls)),

    # Services
    path('', decorate_url_patterns(allianceauth.services.urls.urlpatterns, main_character_required)),

    # Night mode
    path('night/', views.NightModeRedirectView.as_view(), name='nightmode'),

    # Theme Change
    path('theme/', views.ThemeRedirectView.as_view(), name='theme')
]

url_hooks = get_hooks("url_hook")
public_views_allows = getattr(settings, "APPS_WITH_PUBLIC_VIEWS", [])
urlpatterns += urls_from_apps(url_hooks, public_views_allows)



from django.contrib import admin

from .models import DiscourseUser
from ...admin import ServicesUserAdmin


@admin.register(DiscourseUser)
class DiscourseUserAdmin(ServicesUserAdmin):
    list_display = ServicesUserAdmin.list_display + (
        'enabled',
    )



from allianceauth.menu.hooks import MenuItemHook
from . import urls

from allianceauth import hooks
from allianceauth.services.hooks import UrlHook


class PermissionsTool(MenuItemHook):
    def __init__(self):
        MenuItemHook.__init__(
            self,
            'Permissions Audit',
            'fa-solid fa-id-card',
            'permissions_tool:overview',
            order=400,
            navactive=['permissions_tool:']
        )

    def render(self, request):
        if request.user.has_perm('permissions_tool.audit_permissions'):
            return MenuItemHook.render(self, request)
        return ''


@hooks.register('menu_item_hook')
def register_menu():
    return PermissionsTool()


@hooks.register('url_hook')
def register_url():
    return UrlHook(urls, 'permissions_tool', r'^permissions/')



import json
import requests_mock
from unittest.mock import patch

from django.test import RequestFactory, TestCase

from allianceauth.authentication.views import task_counts, esi_check
from allianceauth.tests.auth_utils import AuthUtils
from allianceauth.authentication.constants import ESI_ERROR_MESSAGE_OVERRIDES

MODULE_PATH = "allianceauth.authentication.views"
TEMPLATETAGS_PATH = "allianceauth.templatetags.admin_status"


def jsonresponse_to_dict(response) -> dict:
    return json.loads(response.content)


@patch(MODULE_PATH + ".queued_tasks_count")
@patch(MODULE_PATH + ".active_tasks_count")
@patch(MODULE_PATH + "._celery_stats")
class TestRunningTasksCount(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.factory = RequestFactory()
        cls.user = AuthUtils.create_user("bruce_wayne")
        cls.user.is_superuser = True
        cls.user.save()

    def test_should_return_data(self, mock_celery_stats, mock_tasks_queued, mock_tasks_running):
        # given
        mock_tasks_running.return_value = 2
        mock_tasks_queued.return_value = 3
        mock_celery_stats.return_value = {
            "tasks_succeeded": 5,
            "tasks_retried": 1,
            "tasks_failed": 4,
            "tasks_total": 11,
            "tasks_hours": 24,
            "earliest_task": "2025-08-14T22:47:54.853Z",
        }

        request = self.factory.get("/")
        request.user = self.user

        # when
        response = task_counts(request)

        # then
        self.assertEqual(response.status_code, 200)
        self.assertDictEqual(
            jsonresponse_to_dict(response),
            {
                "tasks_succeeded": 5,
                "tasks_retried": 1,
                "tasks_failed": 4,
                "tasks_total": 11,
                "tasks_hours": 24,
                "earliest_task": "2025-08-14T22:47:54.853Z",
                "tasks_running": 3,
                "tasks_queued": 2,
            }
        )

    def test_su_only(self, mock_celery_stats, mock_tasks_queued, mock_tasks_running):
        self.user.is_superuser = False
        self.user.save()
        self.user.refresh_from_db()

        # given
        mock_tasks_running.return_value = 2
        mock_tasks_queued.return_value = 3
        mock_celery_stats.return_value = {
            "tasks_succeeded": 5,
            "tasks_retried": 1,
            "tasks_failed": 4,
            "tasks_total": 11,
            "tasks_hours": 24,
            "earliest_task": "2025-08-14T22:47:54.853Z",
        }

        request = self.factory.get("/")
        request.user = self.user

        # when
        response = task_counts(request)

        # then
        self.assertEqual(response.status_code, 302)


class TestEsiCheck(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.factory = RequestFactory()
        cls.user = AuthUtils.create_user("bruce_wayne")
        cls.user.is_superuser = True
        cls.user.save()

    @requests_mock.Mocker()
    def test_401_data_returns_200(
        self, m
    ):
        error_json = {
            "error": "You have been banned from using ESI. Please contact Technical Support. (support@eveonline.com)"
        }
        status_code = 401
        m.get(
            "https://esi.evetech.net/latest/status/?datasource=tranquility",
            text=json.dumps(error_json),
            status_code=status_code
        )
        # given
        request = self.factory.get("/")
        request.user = self.user
        # when
        response = esi_check(request)
        # then
        self.assertEqual(response.status_code, 200)
        self.assertDictEqual(
            jsonresponse_to_dict(response), {
                "status": status_code,
                "data": error_json
            }
        )

    @requests_mock.Mocker()
    def test_504_data_returns_200(
        self, m
    ):
        error_json = {
            "error": "Gateway timeout message",
            "timeout": 5000
        }
        status_code = 504
        m.get(
            "https://esi.evetech.net/latest/status/?datasource=tranquility",
            text=json.dumps(error_json),
            status_code=status_code
        )
        # given
        request = self.factory.get("/")
        request.user = self.user
        # when
        response = esi_check(request)
        # then
        self.assertEqual(response.status_code, 200)
        self.assertDictEqual(
            jsonresponse_to_dict(response), {
                "status": status_code,
                "data": error_json
            }
        )

    @requests_mock.Mocker()
    def test_420_data_override(
        self, m
    ):
        error_json = {
            "error": "message from CCP",
        }
        status_code = 420
        m.get(
            "https://esi.evetech.net/latest/status/?datasource=tranquility",
            text=json.dumps(error_json),
            status_code=status_code
        )
        # given
        request = self.factory.get("/")
        request.user = self.user
        # when
        response = esi_check(request)
        # then
        self.assertEqual(response.status_code, 200)
        self.assertNotEqual(
            jsonresponse_to_dict(response)["data"],
            error_json
        )
        self.assertDictEqual(
            jsonresponse_to_dict(response), {
                "status": status_code,
                "data": {
                    "error": ESI_ERROR_MESSAGE_OVERRIDES.get(status_code)
                }
            }
        )

    @requests_mock.Mocker()
    def test_200_data_returns_200(
        self, m
    ):
        good_json = {
            "players": 5,
            "server_version": "69420",
            "start_time": "2030-01-01T23:59:59Z"
        }
        status_code = 200

        m.get(
            "https://esi.evetech.net/latest/status/?datasource=tranquility",
            text=json.dumps(good_json),
            status_code=status_code
        )
        # given
        request = self.factory.get("/")
        request.user = self.user
        # when
        response = esi_check(request)
        # then
        self.assertEqual(response.status_code, 200)
        self.assertDictEqual(
            jsonresponse_to_dict(response), {
                "status": status_code,
                "data": good_json
            }
        )

    def test_su_only(
        self,
    ):
        self.user.is_superuser = False
        self.user.save()
        self.user.refresh_from_db()
        # given
        request = self.factory.get("/")
        request.user = self.user
        # when
        response = esi_check(request)
        # then
        self.assertEqual(response.status_code, 302)



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class SRPConfig(AppConfig):
    name = 'allianceauth.srp'
    label = 'srp'
    verbose_name = _('Ship Replacement')



from django.contrib.auth.models import User
from django.db import models
from django.utils.translation import gettext_lazy as _

from allianceauth.eveonline.models import EveCharacter
from allianceauth.eveonline.models import EveCorporationInfo


class Timer(models.Model):
    class Objective(models.TextChoices):
        """
        Choices for Objective Type
        """

        FRIENDLY = "Friendly", _("Friendly")
        HOSTILE = "Hostile", _("Hostile")
        NEUTRAL = "Neutral", _("Neutral")

    class Structure(models.TextChoices):
        """
        Choices for Structure Type
        """

        POCO = "POCO", _("POCO")
        ORBITALSKYHOOK = "Orbital Skyhook", _("Orbital Skyhook")
        IHUB = "I-HUB", _("Sovereignty Hub")
        TCU = "TCU", _("TCU")  # Pending Remval
        POSS = "POS[S]", _("POS [S]")
        POSM = "POS[M]", _("POS [M]")
        POSL = "POS[L]", _("POS [L]")
        ASTRAHUS = "Astrahus", _("Astrahus")
        FORTIZAR = "Fortizar", _("Fortizar")
        KEEPSTAR = "Keepstar", _("Keepstar")
        RAITARU = "Raitaru", _("Raitaru")
        AZBEL = "Azbel", _("Azbel")
        SOTIYO = "Sotiyo", _("Sotiyo")
        ATHANOR = "Athanor", _("Athanor")
        TATARA = "Tatara", _("Tatara")
        PHAROLUX = "Pharolux Cyno Beacon", _("Cyno Beacon")
        TENEBREX = "Tenebrex Cyno Jammer", _("Cyno Jammer")
        ANSIBLEX = "Ansiblex Jump Gate", _("Ansiblex Jump Gate")
        MERCDEN = "Mercenary Den", _("Mercenary Den")
        MOONPOP = "Moon Mining Cycle", _("Moon Mining Cycle")
        METENOX = "Metenox Moon Drill", _("Metenox Moon Drill")
        OTHER = "Other", _("Other")

    class TimerType(models.TextChoices):
        """
        Choices for Timer Type
        """

        UNSPECIFIED = "UNSPECIFIED", _("Not Specified")
        SHIELD = "SHIELD", _("Shield")
        ARMOR = "ARMOR", _("Armor")
        HULL = "HULL", _("Hull")
        FINAL = "FINAL", _("Final")
        ANCHORING = "ANCHORING", _("Anchoring")
        UNANCHORING = "UNANCHORING", _("Unanchoring")
        ABANDONED = "ABANDONED", _("Abandoned")
        THEFT = "THEFT", _("Theft")

    details = models.CharField(max_length=254, default="")
    system = models.CharField(max_length=254, default="")
    planet_moon = models.CharField(max_length=254, blank=True, default="")
    structure = models.CharField(max_length=254,choices=Structure.choices,default=Structure.OTHER)
    timer_type = models.CharField(max_length=254,choices=TimerType.choices,default=TimerType.UNSPECIFIED)
    objective = models.CharField(max_length=254, choices=Objective.choices, default=Objective.NEUTRAL)
    eve_time = models.DateTimeField()
    important = models.BooleanField(default=False)
    eve_character = models.ForeignKey(EveCharacter, null=True, on_delete=models.SET_NULL)
    eve_corp = models.ForeignKey(EveCorporationInfo, on_delete=models.CASCADE)
    corp_timer = models.BooleanField(default=False)
    user = models.ForeignKey(User, null=True, on_delete=models.SET_NULL)

    def __str__(self) -> str:
        return str(self.system) + ' ' + str(self.details)

    class Meta:
        ordering = ['eve_time']



"""Global constants for the menu app."""

from django.db import models
from django.utils.translation import gettext_lazy as _

DEFAULT_FOLDER_ICON_CLASSES = "fa-solid fa-folder"  # TODO: Make this a setting?
"""Default icon class for folders."""

DEFAULT_MENU_ITEM_ORDER = 9999
"""Default order for any menu item."""


class MenuItemType(models.TextChoices):
    """The type of a menu item."""

    APP = "app", _("app")
    FOLDER = "folder", _("folder")
    LINK = "link", _("link")



from __future__ import annotations

import logging
from typing import TYPE_CHECKING

from django.db import models
from django.db.models import Case, Q, Value, When

from allianceauth.hooks import get_hooks

from .constants import MenuItemType
from .core.menu_item_hooks import MenuItemHookParams, gather_params

if TYPE_CHECKING:
    from .models import MenuItem


logger = logging.getLogger(__name__)


class MenuItemQuerySet(models.QuerySet):
    def filter_folders(self):
        """Add filter to include folders only."""
        return self.filter(hook_hash__isnull=True, url="")

    def annotate_item_type_2(self):
        """Add calculated field with item type."""
        return self.annotate(
            item_type_2=Case(
                When(~Q(hook_hash__isnull=True), then=Value(MenuItemType.APP.value)),
                When(url="", then=Value(MenuItemType.FOLDER.value)),
                default=Value(MenuItemType.LINK.value),
            )
        )


class MenuItemManagerBase(models.Manager):
    def sync_all(self):
        """Sync all menu items from hooks."""
        hook_params = self._gather_menu_item_hook_params()
        self._delete_obsolete_app_items(hook_params)
        self._update_or_create_app_items(hook_params)

    def _gather_menu_item_hook_params(self) -> list[MenuItemHookParams]:
        params = [gather_params(hook()) for hook in get_hooks("menu_item_hook")]
        return params

    def _delete_obsolete_app_items(self, params: list[MenuItemHookParams]):
        hashes = [obj.hash for obj in params]
        self.exclude(hook_hash__isnull=True).exclude(hook_hash__in=hashes).delete()

    def _update_or_create_app_items(self, params: list[MenuItemHookParams]):
        for param in params:
            try:
                obj: MenuItem = self.get(hook_hash=param.hash)
            except self.model.DoesNotExist:
                self.create(hook_hash=param.hash, order=param.order, text=param.text)
            else:
                # if it exists update the text only
                if obj.text != param.text:
                    obj.text = param.text
                    obj.save()

        logger.debug("Updated menu items from %d menu item hooks", len(params))


MenuItemManager = MenuItemManagerBase.from_queryset(MenuItemQuerySet)



import logging

from django.contrib import messages
from django.contrib.auth.mixins import (
    LoginRequiredMixin,
    PermissionRequiredMixin,
)
from django.db.models import Q
from django.shortcuts import get_object_or_404, render
from django.template.loader import render_to_string
from django.urls import reverse_lazy
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django.views import View
from django.views.generic import CreateView, DeleteView, UpdateView

from allianceauth.eveonline.models import EveCorporationInfo
from allianceauth.timerboard.form import TimerForm
from allianceauth.timerboard.models import Timer

logger = logging.getLogger(__name__)

TIMER_VIEW_PERMISSION = "auth.timer_view"
TIMER_MANAGE_PERMISSION = "auth.timer_management"


class BaseTimerView(LoginRequiredMixin, PermissionRequiredMixin, View):
    pass


class TimerView(BaseTimerView):
    template_name = "timerboard/view.html"
    permission_required = TIMER_VIEW_PERMISSION

    def get(self, request):
        """
        Renders the timer view

        :param request:
        :type request:
        :return:
        :rtype:
        """

        def get_bg_modifier(structure):
            """
            Returns the bootstrap bg modifier for the given structure

            :param structure:
            :type structure:
            :return:
            :rtype:
            """

            if structure in bg_info:
                return "info"
            elif structure in bg_warning:
                return "warning"
            elif structure in bg_danger:
                return "danger"
            elif structure in bg_secondary:
                return "secondary"

            return "primary"

        logger.debug(f"timer_view called by user {request.user}")
        char = request.user.profile.main_character

        if char:
            corp = char.corporation
        else:
            corp = None

        base_query = Timer.objects.select_related("eve_character")

        timers = []
        corp_timers = []
        future_timers = []
        past_timers = []

        bg_info = [
            Timer.Structure.POCO.value,  # POCO
            Timer.Structure.POSS.value,  # POS[S]
            Timer.Structure.POSM.value,  # POS[M]
            Timer.Structure.POSL.value,  # POS[L]
        ]
        bg_warning = [
            Timer.Structure.ANSIBLEX.value,  # Ansiblex Jump Gate
            Timer.Structure.ATHANOR.value,  # Athanor
            Timer.Structure.AZBEL.value,  # Azbel
            Timer.Structure.MERCDEN.value,  # Mercenary Den
            Timer.Structure.METENOX.value,  # Metenox Moon Drill
            Timer.Structure.ORBITALSKYHOOK.value,  # Orbital Skyhook
            Timer.Structure.PHAROLUX.value,  # Pharolux Cyno Beacon
            Timer.Structure.RAITARU.value,  # Raitaru
            "Station",  # Legacy structure, remove in future update
            Timer.Structure.TATARA.value,  # Tatara
            Timer.Structure.TENEBREX.value,  # Tenebrex Cyno Jammer
        ]
        bg_danger = [
            Timer.Structure.ASTRAHUS.value,  # Astrahus
            Timer.Structure.FORTIZAR.value,  # Fortizar
            Timer.Structure.IHUB.value,  # I-HUB
            Timer.Structure.KEEPSTAR.value,  # Keepstar
            Timer.Structure.SOTIYO.value,  # Sotiyo
            Timer.Structure.TCU.value,  # TCU (Legacy structure, remove in future update)
        ]
        bg_secondary = [
            Timer.Structure.MOONPOP.value,  # Moon Mining Cycle
            Timer.Structure.OTHER.value,  # Other
        ]

        # Timers
        for timer in base_query.filter(corp_timer=False):
            timer.bg_modifier = get_bg_modifier(timer.structure)
            timers.append(timer)

        # Corp Timers
        for timer in base_query.filter(corp_timer=True, eve_corp=corp):
            timer.bg_modifier = get_bg_modifier(timer.structure)
            corp_timers.append(timer)

        # Future Timers
        for timer in base_query.filter(corp_timer=False, eve_time__gte=timezone.now()):
            timer.bg_modifier = get_bg_modifier(timer.structure)
            future_timers.append(timer)

        # Past Timers
        for timer in base_query.filter(corp_timer=False, eve_time__lt=timezone.now()):
            timer.bg_modifier = get_bg_modifier(timer.structure)
            past_timers.append(timer)

        render_items = {
            "timers": timers,
            "corp_timers": corp_timers,
            "future_timers": future_timers,
            "past_timers": past_timers,
        }

        return render(request, self.template_name, context=render_items)


class TimerManagementView(BaseTimerView):
    permission_required = TIMER_MANAGE_PERMISSION
    index_redirect = "timerboard:view"
    success_url = reverse_lazy(index_redirect)
    model = Timer

    def get_timer(self, timer_id):
        return get_object_or_404(self.model, id=timer_id)


class AddUpdateMixin:
    def get_form_kwargs(self):
        """
        Inject the request user into the kwargs passed to the form
        """
        kwargs = super().get_form_kwargs()
        kwargs.update({"user": self.request.user})
        return kwargs


class AddTimerView(TimerManagementView, AddUpdateMixin, CreateView):
    template_name_suffix = "_create_form"
    form_class = TimerForm

    def form_valid(self, form):
        result = super().form_valid(form)
        timer = self.object
        logger.info(
            f"Created new timer in {timer.system} at {timer.eve_time} by user {self.request.user}"
        )
        messages.success(
            self.request,
            _("Added new timer in %(system)s at %(time)s.")
            % {"system": timer.system, "time": timer.eve_time},
        )
        return result


class EditTimerView(TimerManagementView, AddUpdateMixin, UpdateView):
    template_name_suffix = "_update_form"
    form_class = TimerForm

    def form_valid(self, form):
        messages.success(self.request, _("Saved changes to the timer."))
        return super().form_valid(form)


class RemoveTimerView(TimerManagementView, DeleteView):
    pass


def dashboard_timers(request):
    if request.user.has_perm(TIMER_VIEW_PERMISSION):
        try:
            corp = request.user.profile.main_character.corporation
        except (EveCorporationInfo.DoesNotExist, AttributeError):
            return ""

        timers = Timer.objects.select_related("eve_character").filter(
            (Q(corp_timer=True) & Q(eve_corp=corp)) | Q(corp_timer=False),
            eve_time__gte=timezone.now(),
        )[:5]

        if timers.count():
            context = {
                "timers": timers,
            }

            return render_to_string(
                template_name="timerboard/dashboard.timers.html",
                context=context,
                request=request,
            )
        else:
            return ""
    else:
        return ""



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class DiscourseServiceConfig(AppConfig):
    name = 'allianceauth.services.modules.discourse'
    label = 'discourse'
    verbose_name = _('Discourse Service')



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class AnalyticsConfig(AppConfig):
    name = 'allianceauth.analytics'
    label = 'analytics'
    verbose_name = _('Analytics')



from django.apps import apps
from django.contrib import admin

from django.contrib.auth.models import Group as BaseGroup, Permission, User
from django.db.models import Count, Exists, OuterRef
from django.db.models.functions import Lower
from django.db.models.signals import (
    m2m_changed,
    post_delete,
    post_save,
    pre_delete,
    pre_save
)
from django.dispatch import receiver

from .forms import GroupAdminForm, ReservedGroupNameAdminForm
from .models import AuthGroup, GroupRequest, ReservedGroupName
from .tasks import remove_users_not_matching_states_from_group

if 'eve_autogroups' in apps.app_configs:
    _has_auto_groups = True
else:
    _has_auto_groups = False


class AuthGroupInlineAdmin(admin.StackedInline):
    model = AuthGroup
    filter_horizontal = ('group_leaders', 'group_leader_groups', 'states',)
    fields = (
        'description',
        'group_leaders',
        'group_leader_groups',
        'states',
        'internal',
        'hidden',
        'open',
        'public',
        'restricted',
    )
    verbose_name_plural = 'Auth Settings'
    verbose_name = ''

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        """overriding this formfield to have sorted lists in the form"""
        if db_field.name == "group_leaders":
            kwargs["queryset"] = User.objects.order_by(Lower('username'))
        elif db_field.name == "group_leader_groups":
            kwargs["queryset"] = Group.objects.order_by(Lower('name'))
        return super().formfield_for_manytomany(db_field, request, **kwargs)

    def has_delete_permission(self, request, obj=None):
        return False

    def has_change_permission(self, request, obj=None):
        return request.user.has_perm('auth.change_group')

    def get_readonly_fields(self, request, obj=None):
        if not request.user.is_superuser:
            return self.readonly_fields + ("restricted",)
        return self.readonly_fields


if _has_auto_groups:
    class IsAutoGroupFilter(admin.SimpleListFilter):
        title = 'auto group'
        parameter_name = 'is_auto_group__exact'

        def lookups(self, request, model_admin):
            return (
                ('yes', 'Yes'),
                ('no', 'No'),
            )

        def queryset(self, request, queryset):
            value = self.value()
            if value == 'yes':
                return queryset.exclude(
                    managedalliancegroup__isnull=True,
                    managedcorpgroup__isnull=True
                )
            elif value == 'no':
                return queryset.filter(
                    managedalliancegroup__isnull=True,
                    managedcorpgroup__isnull=True
                )
            return queryset


class HasLeaderFilter(admin.SimpleListFilter):
    title = 'has leader'
    parameter_name = 'has_leader__exact'

    def lookups(self, request, model_admin):
        return (
            ('yes', 'Yes'),
            ('no', 'No'),
        )

    def queryset(self, request, queryset):
        value = self.value()
        if value == 'yes':
            return queryset.filter(authgroup__group_leaders__isnull=False)
        elif value == 'no':
            return queryset.filter(authgroup__group_leaders__isnull=True)
        return queryset


class GroupAdmin(admin.ModelAdmin):
    form = GroupAdminForm
    ordering = ('name',)
    list_display = (
        'name',
        '_description',
        '_properties',
        '_member_count',
        'has_leader',
    )
    list_filter = [
        'authgroup__internal',
        'authgroup__hidden',
        'authgroup__open',
        'authgroup__public',
    ]
    if _has_auto_groups:
        list_filter.append(IsAutoGroupFilter)
    list_filter.append(HasLeaderFilter)

    search_fields = ('name', 'authgroup__description')

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        has_leader_qs = (
            AuthGroup.objects.filter(group=OuterRef('pk'), group_leaders__isnull=False)
        )
        has_leader_groups_qs = (
            AuthGroup.objects.filter(
                group=OuterRef('pk'), group_leader_groups__isnull=False
            )
        )
        qs = (
            qs.select_related('authgroup')
            .annotate(member_count=Count('user', distinct=True))
            .annotate(has_leader=Exists(has_leader_qs))
            .annotate(has_leader_groups=Exists(has_leader_groups_qs))
        )
        if _has_auto_groups:
            is_autogroup_corp = (
                Group.objects.filter(
                    pk=OuterRef('pk'), managedcorpgroup__isnull=False
                )
            )
            is_autogroup_alliance = (
                Group.objects.filter(
                    pk=OuterRef('pk'), managedalliancegroup__isnull=False
                )
            )
            qs = (
                qs.annotate(is_autogroup_corp=Exists(is_autogroup_corp))
                .annotate(is_autogroup_alliance=Exists(is_autogroup_alliance))
            )
        return qs

    def _description(self, obj):
        return obj.authgroup.description

    @admin.display(description='Members', ordering='member_count')
    def _member_count(self, obj):
        return obj.member_count

    @admin.display(boolean=True)
    def has_leader(self, obj):
        return obj.has_leader or obj.has_leader_groups

    def _properties(self, obj):
        properties = list()
        if _has_auto_groups and (obj.is_autogroup_corp or obj.is_autogroup_alliance):
            properties.append('Auto Group')
        elif obj.authgroup.internal:
            properties.append('Internal')
        else:
            if obj.authgroup.hidden:
                properties.append('Hidden')
            if obj.authgroup.open:
                properties.append('Open')
            if obj.authgroup.public:
                properties.append('Public')
        if not properties:
            properties.append('Default')
        if obj.authgroup.restricted:
            properties.append('Restricted')
        return properties

    filter_horizontal = ('permissions',)
    inlines = (AuthGroupInlineAdmin,)

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        if db_field.name == "permissions":
            kwargs["queryset"] = Permission.objects.select_related("content_type").all()
        return super().formfield_for_manytomany(db_field, request, **kwargs)

    def save_formset(self, request, form, formset, change):
        for inline_form in formset:
            ag_instance = inline_form.save(commit=False)
            ag_instance.group = form.instance
            ag_instance.save()
            if ag_instance.states.exists():
                remove_users_not_matching_states_from_group.delay(ag_instance.group.pk)
        formset.save()

    def get_readonly_fields(self, request, obj=None):
        if not request.user.is_superuser:
            return self.readonly_fields + ("permissions",)
        return self.readonly_fields


class Group(BaseGroup):
    class Meta:
        proxy = True
        verbose_name = BaseGroup._meta.verbose_name
        verbose_name_plural = BaseGroup._meta.verbose_name_plural


try:
    admin.site.unregister(BaseGroup)
finally:
    admin.site.register(Group, GroupAdmin)


@admin.register(GroupRequest)
class GroupRequestAdmin(admin.ModelAdmin):
    search_fields = ('user__username', )
    list_display = ('id', 'group', 'user', '_leave_request')
    list_filter = (
        ('group', admin.RelatedOnlyFieldListFilter),
        ('user', admin.RelatedOnlyFieldListFilter),
        'leave_request',
    )

    @admin.display(boolean=True, description="is leave request")
    def _leave_request(self, obj) -> True:
        return obj.leave_request


@admin.register(ReservedGroupName)
class ReservedGroupNameAdmin(admin.ModelAdmin):
    form = ReservedGroupNameAdminForm
    list_display = ("name", "created_by", "created_at")

    def get_form(self, request, *args, **kwargs):
        form = super().get_form(request, *args, **kwargs)
        form.current_user = request.user
        return form

    def has_change_permission(self, *args, **kwargs) -> bool:
        return False


@receiver(pre_save, sender=Group)
def redirect_pre_save(sender, signal=None, *args, **kwargs):
    pre_save.send(BaseGroup, *args, **kwargs)


@receiver(post_save, sender=Group)
def redirect_post_save(sender, signal=None, *args, **kwargs):
    post_save.send(BaseGroup, *args, **kwargs)


@receiver(pre_delete, sender=Group)
def redirect_pre_delete(sender, signal=None, *args, **kwargs):
    pre_delete.send(BaseGroup, *args, **kwargs)


@receiver(post_delete, sender=Group)
def redirect_post_delete(sender, signal=None, *args, **kwargs):
    post_delete.send(BaseGroup, *args, **kwargs)


@receiver(m2m_changed, sender=Group.permissions.through)
def redirect_m2m_changed_permissions(sender, signal=None, *args, **kwargs):
    m2m_changed.send(BaseGroup, *args, **kwargs)



"""
Models for the custom_css app
"""

import os
import re

# Django Solo
from solo.models import SingletonModel

# Django
from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _


class CustomCSS(SingletonModel):
    """
    Model for storing custom CSS for the site
    """

    css = models.TextField(
        blank=True,
        null=True,
        verbose_name=_("Your custom CSS"),
        help_text=_("This CSS will be added to the site after the default CSS."),
    )
    timestamp = models.DateTimeField(auto_now=True)

    class Meta:
        """
        Meta for CustomCSS
        """

        default_permissions = ()
        verbose_name = _("Custom CSS")
        verbose_name_plural = _("Custom CSS")

    def __str__(self) -> str:
        """
        String representation of CustomCSS

        :return:
        :rtype:
        """

        return str(_("Custom CSS"))

    def save(self, *args, **kwargs):
        """
        Save method for CustomCSS

        :param args:
        :type args:
        :param kwargs:
        :type kwargs:
        :return:
        :rtype:
        """

        self.pk = 1

        if self.css and len(self.css.replace(" ", "")) > 0:
            # Write the custom CSS to a file
            custom_css_file = open(
                f"{settings.STATIC_ROOT}allianceauth/custom-styles.css", "w+"
            )
            custom_css_file.write(self.compress_css())
            custom_css_file.close()
        else:
            # Remove the custom CSS file
            try:
                os.remove(f"{settings.STATIC_ROOT}allianceauth/custom-styles.css")
            except FileNotFoundError:
                pass

        super().save(*args, **kwargs)

    def compress_css(self) -> str:
        """
        Compress CSS

        :return:
        :rtype:
        """

        css = self.css
        new_css = ""

        # Remove comments
        css = re.sub(pattern=r"\s*/\*\s*\*/", repl="$$HACK1$$", string=css)
        css = re.sub(pattern=r"/\*[\s\S]*?\*/", repl="", string=css)
        css = css.replace("$$HACK1$$", "/**/")

        # url() doesn't need quotes
        css = re.sub(pattern=r'url\((["\'])([^)]*)\1\)', repl=r"url(\2)", string=css)

        # Spaces may be safely collapsed as generated content will collapse them anyway.
        css = re.sub(pattern=r"\s+", repl=" ", string=css)

        # Shorten collapsable colors: #aabbcc to #abc
        css = re.sub(
            pattern=r"#([0-9a-f])\1([0-9a-f])\2([0-9a-f])\3(\s|;)",
            repl=r"#\1\2\3\4",
            string=css,
        )

        # Fragment values can loose zeros
        css = re.sub(
            pattern=r":\s*0(\.\d+([cm]m|e[mx]|in|p[ctx]))\s*;", repl=r":\1;", string=css
        )

        for rule in re.findall(pattern=r"([^{]+){([^}]*)}", string=css):
            # We don't need spaces around operators
            selectors = [
                re.sub(
                    pattern=r"(?<=[\[\(>+=])\s+|\s+(?=[=~^$*|>+\]\)])",
                    repl=r"",
                    string=selector.strip(),
                )
                for selector in rule[0].split(",")
            ]

            # Order is important, but we still want to discard repetitions
            properties = {}
            porder = []

            for prop in re.findall(pattern="(.*?):(.*?)(;|$)", string=rule[1]):
                key = prop[0].strip().lower()

                if key not in porder:
                    porder.append(key)

                properties[key] = prop[1].strip()

            # output rule if it contains any declarations
            if properties:
                new_css += "{}{{{}}}".format(
                    ",".join(selectors),
                    "".join([f"{key}:{properties[key]};" for key in porder])[:-1],
                )

        return new_css



import tempfile
import webbrowser
from pathlib import Path

from bs4 import BeautifulSoup

from django.http import HttpResponse
from django.template import Context, Template

PACKAGE_PATH = "allianceauth.menu"


def extract_links(response: HttpResponse) -> dict:
    soup = extract_html(response)
    links = {
        link["href"]: "".join(link.stripped_strings)
        for link in soup.find_all("a", href=True)
    }
    return links


def extract_html(response: HttpResponse) -> BeautifulSoup:
    soup = BeautifulSoup(response.content, "html.parser")
    return soup


def open_page_in_browser(response: HttpResponse):
    """Open the response in the system's default browser.

    This will create a temporary file in the user's home.
    """
    path = Path.home() / "temp"
    path.mkdir(exist_ok=True)

    with tempfile.NamedTemporaryFile(dir=path, delete=False) as file:
        file.write(response.content)
        webbrowser.open(file.name)


def render_template(string, context=None):
    context = context or {}
    context = Context(context)
    return Template(string).render(context)


def remove_whitespaces(s) -> str:
    return s.replace("\n", "").strip()



"""
DO NOT EDIT THIS FILE

This settings file contains everything needed for Alliance Auth projects to function.
It gets overwritten by the 'allianceauth update' command.
If you wish to make changes, overload the setting in your project's settings file (local.py).
"""

import os

from celery.schedules import crontab

from django.contrib import messages

from django.utils.translation import gettext_lazy as _

INSTALLED_APPS = [
    "allianceauth",  # needs to be on top of this list to support favicons in Django admin (see https://gitlab.com/allianceauth/allianceauth/-/issues/1301)
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.humanize",
    "django_celery_beat",
    "solo",
    "bootstrapform",
    "django_bootstrap5",  # https://github.com/zostera/django-bootstrap5
    "sortedm2m",
    "esi",
    "allianceauth.framework",
    "allianceauth.authentication",
    "allianceauth.services",
    "allianceauth.eveonline",
    "allianceauth.groupmanagement",
    "allianceauth.notifications",
    "allianceauth.thirdparty.navhelper",
    "allianceauth.analytics",
    "allianceauth.menu",
    "allianceauth.theme",
    "allianceauth.theme.darkly",
    "allianceauth.theme.flatly",
    "allianceauth.theme.materia",
    "allianceauth.custom_css",
    "allianceauth.crontab",
    "sri",
]

SRI_ALGORITHM = "sha512"
SECRET_KEY = "wow I'm a really bad default secret key"

# Celery configuration
BROKER_URL = "redis://localhost:6379/0"
CELERYBEAT_SCHEDULER = "allianceauth.crontab.schedulers.OffsetDatabaseScheduler"
CELERYBEAT_SCHEDULE = {
    "esi_cleanup_callbackredirect": {
        "task": "esi.tasks.cleanup_callbackredirect",
        "schedule": crontab(minute="0", hour="*/4"),
    },
    "esi_cleanup_token": {
        "task": "esi.tasks.cleanup_token",
        "schedule": crontab(minute="0", hour="0"),
        "apply_offset": True,
    },
    "run_model_update": {
        "task": "allianceauth.eveonline.tasks.run_model_update",
        "schedule": crontab(minute="0", hour="*/6"),
        "apply_offset": True,
    },
    "check_all_character_ownership": {
        "task": "allianceauth.authentication.tasks.check_all_character_ownership",
        "schedule": crontab(minute="0", hour="*/4"),
        "apply_offset": True,
    },
    "analytics_daily_stats": {
        "task": "allianceauth.analytics.tasks.analytics_daily_stats",
        "schedule": crontab(minute="0", hour="2"),
    },
}


# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
PROJECT_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
BASE_DIR = os.path.dirname(PROJECT_DIR)

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "allianceauth.authentication.middleware.UserSettingsMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

ROOT_URLCONF = "allianceauth.urls"

LOCALE_PATHS = (os.path.join(BASE_DIR, "locale/"),)

LANGUAGES = (  # Sorted by Language Code alphabetical order + English at top
    ("en", _("English")),
    # ("cs-cz", _("Czech")), #Not yet at 50% translated
    ("de", _("German")),
    ("es", _("Spanish")),
    ("it-it", _("Italian")),
    ("ja", _("Japanese")),
    ("ko-kr", _("Korean")),
    ("fr-fr", _("French")),
    ("nl-nl", _("Dutch")),
    ("pl-pl", _("Polish")),
    ("ru", _("Russian")),
    ("uk", _("Ukrainian")),
    ("zh-hans", _("Simplified Chinese")),
)

# Django's language codes are different from some of the libraries we use,
# so we need to map them.
# When adding a new language, please remember to add it to the mapping
# and add the language files to their respective directories under `allianceauth/static/allianceauth/libs/`.
LANGUAGE_MAPPING = {
    # See https://github.com/DataTables/Plugins/tree/master/i18n for available languages
    # (We use the JSON files)
    # `allianceauth/static/allianceauth/libs/DataTables/Plugins/{version}/i18n/` for the files
    "DataTables": {
        "cs-cz": "cs",
        "de": "de-DE",
        "es": "es-ES",
        "fr-fr": "fr-FR",
        "it-it": "it-IT",
        "ja": "ja",
        "ko-kr": "ko",
        "nl-nl": "nl-NL",
        "pl-pl": "pl",
        "ru": "ru",
        "uk": "uk",
        "zh-hans": "zh-HANT",
    },
    # See https://github.com/moment/moment/tree/master/locale for available languages
    # `allianceauth/static/allianceauth/libs/moment.js/{version}/locale/` for the files
    "MomentJS": {
        "cs-cz": "cs",
        "de": "de",
        "es": "es",
        "fr-fr": "fr",
        "it-it": "it",
        "ja": "ja",
        "ko-kr": "ko",
        "nl-nl": "nl",
        "pl-pl": "pl",
        "ru": "ru",
        "uk": "uk",
        "zh-hans": "zh-cn",
    },
}

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [os.path.join(PROJECT_DIR, "templates")],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "allianceauth.context_processors.auth_settings",
            ],
        },
    },
]

WSGI_APPLICATION = "allianceauth.wsgi.application"

# Password validation
# https://docs.djangoproject.com/en/1.10/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]

AUTHENTICATION_BACKENDS = [
    "allianceauth.authentication.backends.StateBackend",
    "django.contrib.auth.backends.ModelBackend",
]

# Internationalization
# https://docs.djangoproject.com/en/1.10/topics/i18n/

LANGUAGE_CODE = "en-us"

LANGUAGE_COOKIE_AGE = 1209600

TIME_ZONE = "UTC"

USE_I18N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.10/howto/static-files/
STORAGES = {
    "default": {
        "BACKEND": "django.core.files.storage.FileSystemStorage",
    },
    "staticfiles": {
        "BACKEND": "allianceauth.framework.staticfiles.storage.AaManifestStaticFilesStorage",
    },
}

STATIC_URL = "/static/"
STATICFILES_DIRS = [
    os.path.join(PROJECT_DIR, "static"),
]
STATIC_ROOT = os.path.join(BASE_DIR, "static")

# Bootstrap messaging css workaround
MESSAGE_TAGS = {messages.ERROR: "danger error"}

CACHES = {
    "default": {
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://127.0.0.1:6379/1",  # change the 1 here for the DB used
    }
}

SESSION_ENGINE = "django.contrib.sessions.backends.cached_db"

DEBUG = True
DISPLAY_DEBUG = True
ALLOWED_HOSTS = ["*"]
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": str(os.path.join(BASE_DIR, "alliance_auth.sqlite3")),
    },
}

SITE_NAME = "Alliance Auth"

DEFAULT_THEME = "allianceauth.theme.flatly.auth_hooks.FlatlyThemeHook"
DEFAULT_THEME_DARK = "allianceauth.theme.darkly.auth_hooks.DarklyThemeHook"  # Legacy AAv3 user.profile.night_mode=1

LOGIN_URL = "auth_login_user"  # view that handles login logic

LOGIN_REDIRECT_URL = "authentication:dashboard"  # default destination when logging in if no redirect specified
LOGOUT_REDIRECT_URL = "authentication:dashboard"  # destination after logging out
# Both of these redirects accept values as per the django redirect shortcut
# https://docs.djangoproject.com/en/1.11/topics/http/shortcuts/#redirect
# - url names eg 'authentication:dashboard'
# - relative urls eg '/dashboard'
# - absolute urls eg 'http://example.com/dashboard'

# scopes required on new tokens when logging in. Cannot be blank.
LOGIN_TOKEN_SCOPES = ["publicData"]

EMAIL_TIMEOUT = 15

# number of days email verification links are valid for
ACCOUNT_ACTIVATION_DAYS = 1

ESI_API_URL = "https://esi.evetech.net/"

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "verbose": {
            "format": "[%(asctime)s] %(levelname)s [%(name)s:%(lineno)s] %(message)s",
            "datefmt": "%d/%b/%Y %H:%M:%S",
        },
        "simple": {"format": "%(levelname)s %(message)s"},
    },
    "handlers": {
        "log_file": {
            "level": "INFO",  # edit this line to change logging level to file
            "class": "logging.handlers.RotatingFileHandler",
            "filename": os.path.join(BASE_DIR, "log/allianceauth.log"),
            "formatter": "verbose",
            "maxBytes": 1024 * 1024 * 5,  # edit this line to change max log file size
            "backupCount": 5,  # edit this line to change number of log backups
        },
        "extension_file": {
            "level": "INFO",
            "class": "logging.handlers.RotatingFileHandler",
            "filename": os.path.join(BASE_DIR, "log/extensions.log"),
            "formatter": "verbose",
            "maxBytes": 1024 * 1024 * 5,  # edit this line to change max log file size
            "backupCount": 5,  # edit this line to change number of log backups
        },
        "console": {
            "level": "DEBUG",  # edit this line to change logging level to console
            "class": "logging.StreamHandler",
            "formatter": "verbose",
        },
        "notifications": {  # creates notifications for users with logging_notifications permission
            "level": "ERROR",  # edit this line to change logging level to notifications
            "class": "allianceauth.notifications.handlers.NotificationHandler",
            "formatter": "verbose",
        },
    },
    "loggers": {
        "allianceauth": {
            "handlers": ["log_file", "console", "notifications"],
            "level": "DEBUG",
        },
        "extensions": {
            "handlers": ["extension_file", "console"],
            "level": "DEBUG",
        },
        "django": {
            "handlers": ["log_file", "console"],
            "level": "ERROR",
        },
        "esi": {
            "handlers": ["log_file", "console"],
            "level": "DEBUG",
        },
    },
}

DEFAULT_AUTO_FIELD = "django.db.models.AutoField"



import logging

from django.contrib.auth.decorators import login_required
from django.shortcuts import render

from allianceauth.hooks import get_hooks
from .forms import FleetFormatterForm

logger = logging.getLogger(__name__)


@login_required
def fleet_formatter_view(request):
    logger.debug("fleet_formatter_view called by user %s" % request.user)
    generated = ""
    if request.method == 'POST':
        form = FleetFormatterForm(request.POST)
        logger.debug("Received POST request containing form, valid: %s" % form.is_valid())
        if form.is_valid():
            generated = "Fleet Name: " + form.cleaned_data['fleet_name'] + "\n"
            generated = generated + "FC: " + form.cleaned_data['fleet_commander'] + "\n"
            generated = generated + "Comms: " + form.cleaned_data['fleet_comms'] + "\n"
            generated = generated + "Fleet Type: " + form.cleaned_data['fleet_type'] + " || " + form.cleaned_data[
                'ship_priorities'] + "\n"
            generated = generated + "Form Up: " + form.cleaned_data['formup_location'] + " @ " + form.cleaned_data[
                'formup_time'] + "\n"
            generated = generated + "Duration: " + form.cleaned_data['expected_duration'] + "\n"
            generated = generated + "Reimbursable: " + form.cleaned_data['reimbursable'] + "\n"
            generated = generated + "Important: " + form.cleaned_data['important'] + "\n"
            if form.cleaned_data['comments'] != "":
                generated = generated + "Why: " + form.cleaned_data['comments'] + "\n"
            logger.info("Formatted fleet broadcast for user %s" % request.user)
    else:
        form = FleetFormatterForm()
        logger.debug("Returning empty form to user %s" % request.user)

    context = {'form': form, 'generated': generated}

    return render(request, 'services/fleetformattertool.html', context=context)


@login_required
def services_view(request):
    logger.debug("services_view called by user %s" % request.user)
    char = request.user.profile.main_character
    context = {'service_ctrls': []}
    for fn in get_hooks('services_hook'):
        # Render hooked services controls
        svc = fn()
        if svc.show_service_ctrl(request.user):
            context['service_ctrls'].append(svc.render_services_ctrl(request))

    return render(request, 'services/services.html', context=context)


def superuser_test(user):
    return user.is_superuser



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class ExampleServiceConfig(AppConfig):
    name = 'allianceauth.services.modules.example'
    label = 'example_service'
    verbose_name = _('Example Service')



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class EveAutogroupsConfig(AppConfig):
    name = 'allianceauth.eveonline.autogroups'
    label = 'eve_autogroups'
    verbose_name = _('EVE Online Autogroups')

    def ready(self):
        import allianceauth.eveonline.autogroups.signals



from .app_settings import DISCORD_OAUTH_BASE_URL, DISCORD_OAUTH_TOKEN_URL  # noqa
from .client import DiscordClient  # noqa
from .exceptions import (  # noqa
    DiscordApiBackoff,
    DiscordClientException,
    DiscordRateLimitExhausted,
    DiscordTooManyRequestsError,
)
from .helpers import RolesSet  # noqa
from .models import Guild, GuildMember, Role, User  # noqa



"""An auth system for EVE Online to help in-game organizations
manage online service access.
"""

# This will make sure the app is always imported when
# Django starts so that shared_task will use this app.

__version__ = '4.10.0'
__title__ = 'Alliance Auth'
__title_useragent__ = 'AllianceAuth'
__url__ = 'https://gitlab.com/allianceauth/allianceauth'
NAME = f'{__title__} v{__version__}'



"""
Custom template tags for custom_css app
"""

# Alliance Auth Custom CSS
from allianceauth.custom_css.models import CustomCSS

# Django
from django.conf import settings
from django.template.defaulttags import register
from django.utils.safestring import mark_safe

from pathlib import Path


@register.simple_tag
def custom_css_static(path: str) -> str:
    """
    Versioned static URL
    This is to make sure to break the browser cache on CSS updates.

    Example: /static/allianceauth/custom-styles.css?v=1752004819.555084

    :param path:
    :type path:
    :return:
    :rtype:
    """

    try:
        Path(f"{settings.STATIC_ROOT}{path}").resolve(strict=True)
    except FileNotFoundError:
        return ""
    else:
        try:
            custom_css = CustomCSS.objects.get(pk=1)
        except CustomCSS.DoesNotExist:
            return ""
        else:
            custom_css_changed = custom_css.timestamp.timestamp()
            custom_css_version = (
                str(custom_css_changed).replace(" ", "").replace(":", "").replace("-", "")
            )  # remove spaces, colons, and dashes
            versioned_url = f"{settings.STATIC_URL}{path}?v={custom_css_version}"

            return mark_safe(f'<link rel="stylesheet" href="{versioned_url}">')



from django.apps import AppConfig
from django.core.checks import register, Tags
from django.utils.translation import gettext_lazy as _


class AuthenticationConfig(AppConfig):
    name = "allianceauth.authentication"
    label = "authentication"
    verbose_name = _("Authentication")

    def ready(self):
        from allianceauth.authentication import checks, signals  # noqa: F401
        from allianceauth.authentication.task_statistics import (
            signals as celery_signals,
        )

        register(Tags.security)(checks.check_login_scopes_setting)
        celery_signals.reset_counters()



"""Template tags for rendering the new side menu.

Documentation of the render logic
---------------------------------

The are 3 types of menu items:

- App entries: Generated by hooks from Django apps
- Link entries: Linking to external pages. User created.
- Folder: Grouping together several app or link entries. User created.

The MenuItem model holds the current list of all menu items.

App entries are linked to a `MenuItemHook` object in the respective Django app.
Those hook objects contain dynamic logic in a `render()` method,
which must be executed when rendering for the current request.

Since the same template must be used to render all items, link entries and folders
are converted to `MenuItemHookCustom` objects, a sub class of `MenuItemHook`.
This ensures the template only rendered objects of one specific type or sub-type.

The rendered menu items are finally collected in a list of RenderedMenuItem objects,
which is used to render the complete menu.
"""

from dataclasses import dataclass, field
from typing import Dict, List, Optional

from django import template
from django.db.models import QuerySet
from django.http import HttpRequest

from allianceauth.hooks import get_hooks
from allianceauth.menu.core import menu_item_hooks, smart_sync
from allianceauth.menu.models import MenuItem
from allianceauth.services.auth_hooks import MenuItemHook

register = template.Library()


@register.inclusion_tag("menu/menu-block.html", takes_context=True)
def menu_items(context: dict) -> dict:
    """Render menu items for new dashboards."""
    smart_sync.sync_menu()

    items = render_menu(context["request"])
    return {"menu_items": items}


@dataclass
class RenderedMenuItem:
    """A rendered menu item.

    These objects can be rendered with the menu-block template.
    """

    menu_item: MenuItem

    children: List["RenderedMenuItem"] = field(default_factory=list)
    count: Optional[int] = None
    html: str = ""
    html_id: str = ""

    @property
    def is_folder(self) -> bool:
        """Return True if this item is a folder."""
        return self.menu_item.is_folder

    def update_html(self, request: HttpRequest, template: str):
        """Render this menu item with defaults and set HTML ID."""
        hook_obj = self.menu_item.to_hook_obj()
        hook_obj.template = template
        hook_obj.count = self.count
        if self.is_folder:
            hook_obj.children = [child.html for child in self.children]

        self.html = hook_obj.render(request)
        self.html_id = hook_obj.html_id


def render_menu(request: HttpRequest) -> List[RenderedMenuItem]:
    """Return the rendered side menu for including in a template.

    This function is creating BS5 style menus.
    """
    hook_items = _gather_menu_items_from_hooks()

    # Menu items needs to be rendered with the new BS5 template
    bs5_template = "menu/menu-item-bs5.html"

    rendered_items: Dict[int, RenderedMenuItem] = {}
    menu_items: QuerySet[MenuItem] = MenuItem.objects.order_by(
        "parent", "order", "text"
    )
    for item in menu_items:
        if item.is_hidden:
            continue  # do not render hidden items

        if item.is_app_item:
            rendered_item = _render_app_item(request, hook_items, item, bs5_template)
            if rendered_item.html == "":
                # If there is no content dont render it.
                # This item has probably been hidden by permissions
                continue
        elif item.is_link_item:
            rendered_item = _render_link_item(request, item, bs5_template)
        elif item.is_folder:
            rendered_item = RenderedMenuItem(item)  # we render these items later
        else:
            raise NotImplementedError("Unknown menu item type")

        if item.is_child:
            try:
                parent = rendered_items[item.parent_id]
            except KeyError:
                continue  # do not render children of hidden folders

            parent.children.append(rendered_item)
            if rendered_item.count is not None:
                if parent.count is None:
                    parent.count = 0
                parent.count += rendered_item.count

        else:
            rendered_items[item.id] = rendered_item

    _remove_empty_folders(rendered_items)

    _render_folder_items(request, rendered_items, bs5_template)

    return list(rendered_items.values())


def _gather_menu_items_from_hooks() -> Dict[str, MenuItemHook]:
    hook_items = {}
    for hook in get_hooks("menu_item_hook"):
        f = hook()
        hook_items[menu_item_hooks.generate_hash(f)] = f
    return hook_items


def _render_app_item(
    request: HttpRequest, hook_items: dict, item: MenuItem, new_template: str
) -> RenderedMenuItem:
    # This is a module hook, so we need to render it as the developer intended
    # TODO add a new attribute for apps that want to override it in the new theme
    hook_item = hook_items[item.hook_hash]
    hook_item.template = new_template
    html = hook_item.render(request)
    count = hook_item.count
    rendered_item = RenderedMenuItem(menu_item=item, count=count, html=html)
    return rendered_item


def _render_link_item(
    request: HttpRequest, item: MenuItem, new_template: str
) -> RenderedMenuItem:
    rendered_item = RenderedMenuItem(menu_item=item)
    rendered_item.update_html(request, template=new_template)
    return rendered_item


def _render_folder_items(
    request: HttpRequest, rendered_items: Dict[int, RenderedMenuItem], new_template: str
):
    for item in rendered_items.values():
        if item.menu_item.is_folder:
            item.update_html(request=request, template=new_template)


def _remove_empty_folders(rendered_items: Dict[int, RenderedMenuItem]):
    ids_to_remove = []
    for item_id, item in rendered_items.items():
        if item.is_folder and not item.children:
            ids_to_remove.append(item_id)

    for item_id in ids_to_remove:
        del rendered_items[item_id]



import logging

from django.conf import settings
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render
from django.utils.translation import gettext_lazy as _
from django.views.generic.base import View

logger = logging.getLogger(__name__)


class NightModeRedirectView(View):
    SESSION_VAR = "NIGHT_MODE"

    def get(self, request, *args, **kwargs):
        request.session[self.SESSION_VAR] = not self.night_mode_state(request)
        if not request.user.is_anonymous:
            try:
                request.user.profile.night_mode = request.session[self.SESSION_VAR]
                request.user.profile.save()
            except Exception as e:
                logger.exception(e)

        return HttpResponseRedirect(request.GET.get("next", "/"))

    @classmethod
    def night_mode_state(cls, request):
        try:
            return request.session.get(cls.SESSION_VAR, False)
        except AttributeError:
            # Session is middleware
            # Sometimes request wont have a session attribute
            return False


class ThemeRedirectView(View):
    THEME_VAR = "THEME"

    def post(self, request, *args, **kwargs):
        theme = request.POST.get('theme', settings.DEFAULT_THEME)
        if not request.user.is_anonymous:
            try:
                request.user.profile.theme = theme
                request.user.profile.save()
                request.session[self.THEME_VAR] = theme
            except Exception as e:
                logger.exception(e)

        return HttpResponseRedirect(request.GET.get("next", "/"))


# TODO: error views should be renamed to a proper function name when possible

def Generic400Redirect(request, *args, **kwargs):
    title = _("Bad Request")
    message = _(
        "Auth encountered an error processing your request, please try again. "
        "If the error persists, please contact the administrators."
    )
    response = _build_error_response(request, title, message, 400)
    return response


def Generic403Redirect(request, *args, **kwargs):
    title = _("Permission Denied")
    message = _(
        "You do not have permission to access the requested page. "
        "If you believe this is in error please contact the administrators."
    )
    response = _build_error_response(request, title, message, 403)
    return response


def Generic404Redirect(request, *args, **kwargs):
    title = _("Page Not Found")
    message = _(
        "Page does not exist. "
        "If you believe this is in error please contact the administrators. "
    )
    response = _build_error_response(request, title, message, 404)
    return response


def Generic500Redirect(request, *args, **kwargs):
    title = _("Internal Server Error")
    message = _(
        "Auth encountered an error processing your request, please try again. "
        "If the error persists, please contact the administrators."
    )
    response = _build_error_response(request, title, message, 500)
    return response


def _build_error_response(request, title, message, status_code) -> HttpResponse:
    context = {"error_title": title, "error_message": message}
    response = render(request, "allianceauth/error.html", context)
    response.status_code = status_code
    return response



# Generated by Django 1.11.5 on 2017-09-28 02:16

import django.contrib.auth.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
        ('groupmanagement', '0006_request_groups_perm'),
    ]

    operations = [
        migrations.CreateModel(
            name='Group',
            fields=[
            ],
            options={
                'verbose_name': 'group',
                'indexes': [],
                'proxy': True,
                'verbose_name_plural': 'groups',
            },
            bases=('auth.group',),
            managers=[
                ('objects', django.contrib.auth.models.GroupManager()),
            ],
        ),
        migrations.RemoveField(
            model_name='grouprequest',
            name='main_char',
        ),
    ]



import logging
from typing import ClassVar

from django.contrib.auth.models import User, Permission
from django.db import models, transaction
from django.utils.translation import gettext_lazy as _
from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo, EveAllianceInfo, EveFactionInfo
from allianceauth.notifications import notify
from django.conf import settings

from .managers import CharacterOwnershipManager, StateManager

logger = logging.getLogger(__name__)


class State(models.Model):
    name = models.CharField(max_length=32, unique=True)
    permissions = models.ManyToManyField(Permission, blank=True)
    priority = models.IntegerField(unique=True, help_text="Users get assigned the state with the highest priority available to them.")

    member_characters = models.ManyToManyField(EveCharacter, blank=True,
                                                help_text="Characters to which this state is available.")
    member_corporations = models.ManyToManyField(EveCorporationInfo, blank=True,
                                                help_text="Corporations to whose members this state is available.")
    member_alliances = models.ManyToManyField(EveAllianceInfo, blank=True,
                                            help_text="Alliances to whose members this state is available.")
    member_factions = models.ManyToManyField(EveFactionInfo, blank=True,
                                            help_text="Factions to whose members this state is available.")
    public = models.BooleanField(default=False, help_text="Make this state available to any character.")

    objects: ClassVar[StateManager] = StateManager()

    class Meta:
        ordering = ['-priority']

    def __str__(self):
        return self.name

    def available_to_character(self, character):
        return self in State.objects.available_to_character(character)

    def available_to_user(self, user):
        return self in State.objects.available_to_user(user)

    def delete(self, **kwargs):
        with transaction.atomic():
            for profile in self.userprofile_set.all():
                profile.assign_state(state=State.objects.exclude(pk=self.pk).get_for_user(profile.user))
        super().delete(**kwargs)


def get_guest_state():
    try:
        return State.objects.get(name='Guest')
    except State.DoesNotExist:
        return State.objects.create(name='Guest', priority=0, public=True)


def get_guest_state_pk():
    return get_guest_state().pk


class UserProfile(models.Model):
    class Meta:
        default_permissions = ('change',)

    class Language(models.TextChoices):
        """
        Choices for UserProfile.language
        """
        # Sorted by Language Code alphabetical order + English at top
        ENGLISH = 'en', _('English')
        CZECH = 'cs-cz', _("Czech")  # Not yet at 50% translated
        GERMAN = 'de', _('German')
        SPANISH = 'es', _('Spanish')
        ITALIAN = 'it-it', _('Italian')
        JAPANESE = 'ja', _('Japanese')
        KOREAN = 'ko-kr', _('Korean')
        FRENCH = 'fr-fr', _('French')
        RUSSIAN = 'ru', _('Russian')
        DUTCH = 'nl-nl', _("Dutch")
        POLISH = 'pl-pl', _("Polish")
        UKRAINIAN = 'uk', _('Ukrainian')
        CHINESE = 'zh-hans', _('Simplified Chinese')

    user = models.OneToOneField(
        User,
        related_name='profile',
        on_delete=models.CASCADE)
    main_character = models.OneToOneField(
        EveCharacter,
        blank=True,
        null=True,
        on_delete=models.SET_NULL)
    state = models.ForeignKey(
        State,
        on_delete=models.SET_DEFAULT,
        default=get_guest_state_pk)
    language = models.CharField(
        _("Language"), max_length=10,
        choices=Language.choices,
        blank=True,
        default='')
    night_mode = models.BooleanField(
        _("Night Mode"),
        blank=True,
        null=True)
    theme = models.CharField(
        _("Theme"),
        max_length=200,
        blank=True,
        null=True,
        help_text="Bootstrap 5 Themes from https://bootswatch.com/ or Community Apps"
    )

    def assign_state(self, state=None, commit=True):
        if not state:
            state = State.objects.get_for_user(self.user)
        if self.state != state:
            self.state = state
            if commit:
                logger.info(f'Updating {self.user} state to {self.state}')
                self.save(update_fields=['state'])
                notify(
                    self.user,
                    _('State changed to: %s' % state),
                    _('Your user\'s state is now: %(state)s')
                    % ({'state': state}),
                    'info'
                )
                from allianceauth.authentication.signals import state_changed

                # We need to ensure we get up to date perms here as they will have just changed.
                # Clear all attribute caches and reload the model that will get passed to the signals!
                self.refresh_from_db()

                state_changed.send(
                    sender=self.__class__, user=self.user, state=self.state
                )

    def __str__(self) -> str:
        return str(self.user)


class CharacterOwnership(models.Model):
    class Meta:
        default_permissions = ('change', 'delete')
        ordering = ['user', 'character__character_name']

    character = models.OneToOneField(EveCharacter, on_delete=models.CASCADE, related_name='character_ownership')
    owner_hash = models.CharField(max_length=28, unique=True)
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name='character_ownerships')

    objects: ClassVar[CharacterOwnershipManager] = CharacterOwnershipManager()

    def __str__(self):
        return f"{self.user}: {self.character}"


class OwnershipRecord(models.Model):
    character = models.ForeignKey(EveCharacter, on_delete=models.CASCADE, related_name='ownership_records')
    owner_hash = models.CharField(max_length=28, db_index=True)
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name='ownership_records')
    created = models.DateTimeField(auto_now=True)

    class Meta:
        ordering = ['-created']

    def __str__(self):
        return f"{self.user}: {self.character} on {self.created}"



"""
Admin classes for custom_css app
"""

# Django
from django.contrib import admin

# Django Solos
from solo.admin import SingletonModelAdmin

# Alliance Auth Custom CSS
from allianceauth.custom_css.models import CustomCSS
from allianceauth.custom_css.forms import CustomCSSAdminForm


@admin.register(CustomCSS)
class CustomCSSAdmin(SingletonModelAdmin):
    """
    Custom CSS Admin
    """

    form = CustomCSSAdminForm

    # Leave this here for when we decide to add syntax highlighting to the CSS editor
    # change_form_template = 'custom_css/admin/change_form.html'



# myapp/tests/test_tasks.py

import logging
from unittest.mock import patch
from django.test import TestCase
from django.db import ProgrammingError
from celery.schedules import crontab

from allianceauth.crontab.utils import offset_cron
from allianceauth.crontab.models import CronOffset

logger = logging.getLogger(__name__)


class TestOffsetCron(TestCase):

    def test_offset_cron_normal(self):
        """
        Test that offset_cron modifies the minute/hour fields
        based on the CronOffset values when everything is normal.
        """
        # We'll create a mock CronOffset instance
        mock_offset = CronOffset(minute=0.5, hour=0.5)

        # Our initial crontab schedule
        original_schedule = crontab(
            minute=[0, 5, 55],
            hour=[0, 3, 23],
            day_of_month='*',
            month_of_year='*',
            day_of_week='*'
        )

        # Patch CronOffset.get_solo to return our mock offset
        with patch('allianceauth.crontab.models.CronOffset.get_solo', return_value=mock_offset):
            new_schedule = offset_cron(original_schedule)

        # Check the new minute/hour
        # minute 0 -> 0 + round(60 * 0.5) = 30 % 60 = 30
        # minute 5 -> 5 + 30 = 35 % 60 = 35
        # minute 55 -> 55 + 30 = 85 % 60 = 25  --> sorted => 25,30,35
        self.assertEqual(new_schedule._orig_minute, '25,30,35')

        # hour 0 -> 0 + round(24 * 0.5) = 12 % 24 = 12
        # hour 3 -> 3 + 12 = 15 % 24 = 15
        # hour 23 -> 23 + 12 = 35 % 24 = 11 --> sorted => 11,12,15
        self.assertEqual(new_schedule._orig_hour, '11,12,15')

        # Check that other fields are unchanged
        self.assertEqual(new_schedule._orig_day_of_month, '*')
        self.assertEqual(new_schedule._orig_month_of_year, '*')
        self.assertEqual(new_schedule._orig_day_of_week, '*')

    def test_offset_cron_programming_error(self):
        """
        Test that if a ProgrammingError is raised (e.g. before migrations),
        offset_cron just returns the original schedule.
        """
        original_schedule = crontab(minute=[0, 15, 30], hour=[1, 2, 3])

        # Force get_solo to raise ProgrammingError
        with patch('allianceauth.crontab.models.CronOffset.get_solo', side_effect=ProgrammingError()):
            new_schedule = offset_cron(original_schedule)

        # Should return the original schedule unchanged
        self.assertEqual(new_schedule, original_schedule)

    def test_offset_cron_unexpected_exception(self):
        """
        Test that if any other exception is raised, offset_cron
        also returns the original schedule, and logs the error.
        """
        original_schedule = crontab(minute='0', hour='0')

        # Force get_solo to raise a generic Exception
        with patch('allianceauth.crontab.models.CronOffset.get_solo', side_effect=Exception("Something bad")):
            new_schedule = offset_cron(original_schedule)

        # Should return the original schedule unchanged
        self.assertEqual(new_schedule, original_schedule)



import logging

from django.db import models
from . import providers

logger = logging.getLogger(__name__)


class EveCharacterProviderManager:
    def get_character(self, character_id) -> providers.Character:
        return providers.provider.get_character(character_id)


class EveCharacterManager(models.Manager):
    provider = EveCharacterProviderManager()

    def exclude_biomassed(self):
        """
        Get a queryset of EveCharacter objects, excluding the "Doomheim" corporation (1000001).

        :return:
        :rtype:
        """

        return self.exclude(corporation_id=1000001)

    def create_character(self, character_id) -> models.Model:
        return self.create_character_obj(self.provider.get_character(character_id))

    def create_character_obj(self, character: providers.Character) -> models.Model:
        return self.create(
            character_id=character.id,
            character_name=character.name,
            corporation_id=character.corp.id,
            corporation_name=character.corp.name,
            corporation_ticker=character.corp.ticker,
            alliance_id=character.alliance.id,
            alliance_name=character.alliance.name,
            alliance_ticker=getattr(character.alliance, 'ticker', None),
            faction_id=character.faction.id,
            faction_name=character.faction.name
        )

    def update_character(self, character_id):
        return self.get(character_id=character_id).update_character()

    def get_character_by_id(self, character_id: int):
        """Return character by character ID or None if not found."""
        try:
            return self.get(character_id=character_id)
        except self.model.DoesNotExist:
            return None


class EveAllianceProviderManager:
    def get_alliance(self, alliance_id) -> providers.Alliance:
        return providers.provider.get_alliance(alliance_id)


class EveAllianceManager(models.Manager):
    provider = EveAllianceProviderManager()

    def create_alliance(self, alliance_id):
        obj = self.create_alliance_obj(self.provider.get_alliance(alliance_id))
        obj.populate_alliance()
        return obj

    def create_alliance_obj(self, alliance: providers.Alliance):
        return self.create(
            alliance_id=alliance.id,
            alliance_name=alliance.name,
            alliance_ticker=alliance.ticker,
            executor_corp_id=alliance.executor_corp_id,
        )

    def update_alliance(self, alliance_id):
        return self.get(alliance_id=alliance_id).update_alliance()


class EveCorporationProviderManager:
    def get_corporation(self, corp_id) -> providers.Corporation:
        return providers.provider.get_corp(corp_id)


class EveCorporationManager(models.Manager):
    provider = EveCorporationProviderManager()

    def create_corporation(self, corp_id):
        return self.create_corporation_obj(self.provider.get_corporation(corp_id))

    def create_corporation_obj(self, corp: providers.Corporation):
        from .models import EveAllianceInfo
        try:
            alliance = EveAllianceInfo.objects.get(alliance_id=corp.alliance_id)
        except EveAllianceInfo.DoesNotExist:
            alliance = None
        return self.create(
            corporation_id=corp.id,
            corporation_name=corp.name,
            corporation_ticker=corp.ticker,
            member_count=corp.members,
            alliance=alliance,
        )

    def update_corporation(self, corp_id):
        return self\
            .get(corporation_id=corp_id)\
            .update_corporation(self.provider.get_corporation(corp_id))



from unittest.mock import patch

from django.test import RequestFactory, TestCase

from allianceauth.menu.templatetags.menu_items import render_menu
from allianceauth.menu.tests.factories import create_menu_item_hook_function
from allianceauth.menu.tests.utils import PACKAGE_PATH, render_template

MODULE_PATH = PACKAGE_PATH + ".templatetags.menu_items"


class TestTemplateTags(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.factory = RequestFactory()

    @patch(MODULE_PATH + ".render_menu", spec=True)
    def test_menu_items(self, mock_render_menu):
        # given
        mock_render_menu.return_value = ["Alpha"]
        request = self.factory.get("/")

        # when
        rendered = render_template(
            "{% load menu_items %}{% menu_items %}",
            context={"request": request},
        )
        self.assertIn("Alpha", rendered)


@patch(MODULE_PATH + ".get_hooks", spec=True)
class TestRenderMenu(TestCase):
    def setUp(self) -> None:
        self.factory = RequestFactory()

    def test_should_render_menu_in_order(self, mock_get_hooks):
        # given
        mock_get_hooks.return_value = [
            create_menu_item_hook_function(text="Charlie"),
            create_menu_item_hook_function(text="Alpha", order=1),
            create_menu_item_hook_function(text="Bravo", order=2),
        ]
        request = self.factory.get("/")

        # when
        result = render_menu(request)

        # then
        menu = list(result)
        self.assertEqual(len(menu), 3)
        self.assertIn("Alpha", menu[0])
        self.assertIn("Bravo", menu[1])
        self.assertIn("Charlie", menu[2])



from unittest.mock import patch

from django.test import TestCase, TransactionTestCase, override_settings

from ..models import EveAllianceInfo, EveCharacter, EveCorporationInfo
from ..tasks import (
    run_model_update,
    update_alliance,
    update_character,
    update_character_chunk,
    update_corp,
)
from .esi_client_stub import EsiClientStub


@patch('allianceauth.eveonline.providers.esi_client_factory')
class TestUpdateTasks(TestCase):
    def test_should_update_alliance(self, mock_esi_client_factory):
        # given
        mock_esi_client_factory.return_value = EsiClientStub()
        my_alliance = EveAllianceInfo.objects.create(
            alliance_id=3001,
            alliance_name="Wayne Enterprises",
            alliance_ticker="WYE",
            executor_corp_id=2003
        )
        # when
        update_alliance(my_alliance.alliance_id)
        # then
        my_alliance.refresh_from_db()
        self.assertEqual(my_alliance.executor_corp_id, 2001)

    def test_should_update_character(self, mock_esi_client_factory):
        # given
        mock_esi_client_factory.return_value = EsiClientStub()
        my_character = EveCharacter.objects.create(
            character_id=1001,
            character_name="Bruce Wayne",
            corporation_id=2002,
            corporation_name="Wayne Food",
            corporation_ticker="WYF",
            alliance_id=None
        )
        # when
        update_character(my_character.character_id)
        # then
        my_character.refresh_from_db()
        self.assertEqual(my_character.corporation_id, 2001)

    def test_should_update_corp(self, mock_esi_client_factory):
        # given
        mock_esi_client_factory.return_value = EsiClientStub()
        EveAllianceInfo.objects.create(
            alliance_id=3001,
            alliance_name="Wayne Enterprises",
            alliance_ticker="WYE",
            executor_corp_id=2003
        )
        my_corporation = EveCorporationInfo.objects.create(
            corporation_id=2003,
            corporation_name="Wayne Food",
            corporation_ticker="WFO",
            member_count=1,
            alliance=None,
            ceo_id=1999
        )
        # when
        update_corp(my_corporation.corporation_id)
        # then
        my_corporation.refresh_from_db()
        self.assertEqual(my_corporation.alliance.alliance_id, 3001)

    # @patch('allianceauth.eveonline.tasks.EveCharacter')
    # def test_update_character(self, mock_EveCharacter):
    #     update_character(42)
    #     self.assertEqual(
    #         mock_EveCharacter.objects.update_character.call_count, 1
    #     )
    #     self.assertEqual(
    #         mock_EveCharacter.objects.update_character.call_args[0][0], 42
    #     )


@override_settings(CELERY_ALWAYS_EAGER=True)
@patch('allianceauth.eveonline.providers.esi_client_factory')
@patch('allianceauth.eveonline.tasks.providers')
@patch('allianceauth.eveonline.tasks.CHARACTER_AFFILIATION_CHUNK_SIZE', 2)
class TestRunModelUpdate(TransactionTestCase):
    def test_should_run_updates(self, mock_providers, mock_esi_client_factory):
        # given
        mock_providers.provider.client = EsiClientStub()
        mock_esi_client_factory.return_value = EsiClientStub()
        EveCorporationInfo.objects.create(
            corporation_id=2001,
            corporation_name="Wayne Technologies",
            corporation_ticker="WTE",
            member_count=10,
            alliance=None,
        )
        alliance_3001 = EveAllianceInfo.objects.create(
            alliance_id=3001,
            alliance_name="Wayne Enterprises",
            alliance_ticker="WYE",
            executor_corp_id=2003
        )
        corporation_2003 = EveCorporationInfo.objects.create(
            corporation_id=2003,
            corporation_name="Wayne Energy",
            corporation_ticker="WEG",
            member_count=99,
            alliance=None,
        )
        character_1001 = EveCharacter.objects.create(
            character_id=1001,
            character_name="Bruce Wayne",
            corporation_id=2002,
            corporation_name="Wayne Food",
            corporation_ticker="WYF",
            alliance_id=None
        )
        # when
        run_model_update()
        # then
        character_1001.refresh_from_db()
        self.assertEqual(
            character_1001.corporation_id, 2001  # char has new corp
        )
        corporation_2003.refresh_from_db()
        self.assertEqual(
            corporation_2003.alliance.alliance_id, 3001  # corp has new alliance
        )
        alliance_3001.refresh_from_db()
        self.assertEqual(
            alliance_3001.executor_corp_id, 2001  # alliance has been updated
        )


@override_settings(CELERY_ALWAYS_EAGER=True)
@patch('allianceauth.eveonline.tasks.update_character', wraps=update_character)
@patch('allianceauth.eveonline.providers.esi_client_factory')
@patch('allianceauth.eveonline.tasks.providers')
@patch('allianceauth.eveonline.tasks.CHARACTER_AFFILIATION_CHUNK_SIZE', 2)
class TestUpdateCharacterChunk(TestCase):
    @staticmethod
    def _updated_character_ids(spy_update_character) -> set:
        """Character IDs passed to update_character task for update."""
        return {
            x[1]["args"][0] for x in spy_update_character.apply_async.call_args_list
        }

    def test_should_update_corp_change(
        self, mock_providers, mock_esi_client_factory, spy_update_character
    ):
        # given
        mock_providers.provider.client = EsiClientStub()
        mock_esi_client_factory.return_value = EsiClientStub()
        character_1001 = EveCharacter.objects.create(
            character_id=1001,
            character_name="Bruce Wayne",
            corporation_id=2003,
            corporation_name="Wayne Energy",
            corporation_ticker="WEG",
            alliance_id=3001,
            alliance_name="Wayne Enterprises",
            alliance_ticker="WYE",
        )
        character_1002 = EveCharacter.objects.create(
            character_id=1002,
            character_name="Peter Parker",
            corporation_id=2001,
            corporation_name="Wayne Technologies",
            corporation_ticker="WTE",
            alliance_id=3001,
            alliance_name="Wayne Enterprises",
            alliance_ticker="WYE",
        )
        # when
        update_character_chunk([
            character_1001.character_id, character_1002.character_id
        ])
        # then
        character_1001.refresh_from_db()
        self.assertEqual(character_1001.corporation_id, 2001)
        self.assertSetEqual(self._updated_character_ids(spy_update_character), {1001})

    def test_should_update_name_change(
        self, mock_providers, mock_esi_client_factory, spy_update_character
    ):
        # given
        mock_providers.provider.client = EsiClientStub()
        mock_esi_client_factory.return_value = EsiClientStub()
        character_1001 = EveCharacter.objects.create(
            character_id=1001,
            character_name="Batman",
            corporation_id=2001,
            corporation_name="Wayne Technologies",
            corporation_ticker="WTE",
            alliance_id=3001,
            alliance_name="Wayne Technologies",
            alliance_ticker="WYT",
        )
        # when
        update_character_chunk([character_1001.character_id])
        # then
        character_1001.refresh_from_db()
        self.assertEqual(character_1001.character_name, "Bruce Wayne")
        self.assertSetEqual(self._updated_character_ids(spy_update_character), {1001})

    def test_should_update_alliance_change(
        self, mock_providers, mock_esi_client_factory, spy_update_character
    ):
        # given
        mock_providers.provider.client = EsiClientStub()
        mock_esi_client_factory.return_value = EsiClientStub()
        character_1001 = EveCharacter.objects.create(
            character_id=1001,
            character_name="Bruce Wayne",
            corporation_id=2001,
            corporation_name="Wayne Technologies",
            corporation_ticker="WTE",
            alliance_id=None,
        )
        # when
        update_character_chunk([character_1001.character_id])
        # then
        character_1001.refresh_from_db()
        self.assertEqual(character_1001.alliance_id, 3001)
        self.assertSetEqual(self._updated_character_ids(spy_update_character), {1001})

    def test_should_not_update_when_not_changed(
        self, mock_providers, mock_esi_client_factory, spy_update_character
    ):
        # given
        mock_providers.provider.client = EsiClientStub()
        mock_esi_client_factory.return_value = EsiClientStub()
        character_1001 = EveCharacter.objects.create(
            character_id=1001,
            character_name="Bruce Wayne",
            corporation_id=2001,
            corporation_name="Wayne Technologies",
            corporation_ticker="WTE",
            alliance_id=3001,
            alliance_name="Wayne Technologies",
            alliance_ticker="WYT",
        )
        # when
        update_character_chunk([character_1001.character_id])
        # then
        self.assertSetEqual(self._updated_character_ids(spy_update_character), set())

    def test_should_fall_back_to_single_updates_when_bulk_update_failed(
        self, mock_providers, mock_esi_client_factory, spy_update_character
    ):
        # given
        mock_providers.provider.client.Character.post_characters_affiliation\
            .side_effect = OSError
        mock_esi_client_factory.return_value = EsiClientStub()
        character_1001 = EveCharacter.objects.create(
            character_id=1001,
            character_name="Bruce Wayne",
            corporation_id=2001,
            corporation_name="Wayne Technologies",
            corporation_ticker="WTE",
            alliance_id=3001,
            alliance_name="Wayne Technologies",
            alliance_ticker="WYT",
        )
        # when
        update_character_chunk([character_1001.character_id])
        # then
        self.assertSetEqual(self._updated_character_ids(spy_update_character), {1001})



# Generated by Django 1.11.5 on 2017-09-28 02:16

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0008_remove_apikeys'),
    ]

    operations = [
        migrations.AlterField(
            model_name='evecorporationinfo',
            name='alliance',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='eveonline.EveAllianceInfo'),
        ),
    ]



# Generated by Django 3.1.1 on 2021-01-05 14:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0013_evecorporationinfo_ceo_id'),
    ]

    operations = [
        migrations.AddIndex(
            model_name='evecorporationinfo',
            index=models.Index(fields=['ceo_id'], name='eveonline_e_ceo_id_eea7b8_idx'),
        ),
    ]



# Generated by Django 3.1.4 on 2020-12-30 08:53

from uuid import uuid4
from django.db import migrations


def generate_identifier(apps, schema_editor):
    # We can't import the Person model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    Identifier = apps.get_model('analytics', 'AnalyticsIdentifier')
    identifier = Identifier()
    identifier.id = 1
    identifier.save()


def zero_identifier(apps, schema_editor):
    # Have to define some code to remove this identifier
    # In case of migration rollback?
    Identifier = apps.get_model('analytics', 'AnalyticsIdentifier')
    Identifier.objects.filter(id=1).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('analytics', '0002_add_AA_Team_Token'),
    ]

    operations = [migrations.RunPython(generate_identifier, zero_identifier)
                    ]



from . import (
    _ESI_CATEGORY_ALLIANCE,
    _ESI_CATEGORY_CHARACTER,
    _ESI_CATEGORY_CORPORATION,
    _ESI_CATEGORY_INVENTORYTYPE
)


_EVE_IMAGE_SERVER_URL = 'https://images.evetech.net'
_DEFAULT_IMAGE_SIZE = 32


def _eve_entity_image_url(
    category: str,
    entity_id: int,
    size: int = 32,
    variant: str = None,
    tenant: str = None,
) -> str:
    """returns image URL for an Eve Online ID.
    Supported categories: alliance, corporation, character, inventory_type

    Arguments:
    - category: category of the ID, see ESI category constants
    - entity_id: Eve ID of the entity
    - size: (optional) render size of the image.must be between 32 (default) and 1024
    - variant: (optional) image variant for category. currently not relevant.
    - tenant: (optional) Eve Server, either `tranquility`(default) or `singularity`

    Returns:
    - URL string for the requested image on the Eve image server

    Exceptions:
    - Throws ValueError on invalid input
    """

    # input validations
    categories = {
        _ESI_CATEGORY_ALLIANCE: {
            'endpoint': 'alliances',
            'variants': ['logo']
        },
        _ESI_CATEGORY_CORPORATION: {
            'endpoint': 'corporations',
            'variants': ['logo']
        },
        _ESI_CATEGORY_CHARACTER: {
            'endpoint': 'characters',
            'variants': ['portrait']
        },
        _ESI_CATEGORY_INVENTORYTYPE: {
            'endpoint': 'types',
            'variants': ['icon', 'render']
        }
    }
    tenants = ['tranquility', 'singularity']

    if not entity_id:
        raise ValueError(f'Invalid entity_id: {entity_id}')
    else:
        entity_id = int(entity_id)

    if not size or size < 32 or size > 1024 or (size & (size - 1) != 0):
        raise ValueError(f'Invalid size: {size}')

    if category not in categories:
        raise ValueError(f'Invalid category {category}')
    else:
        endpoint = categories[category]['endpoint']

    if variant:
        if variant not in categories[category]['variants']:
            raise ValueError('Invalid variant {} for category {}'.format(
                variant,
                category
            ))
    else:
        variant = categories[category]['variants'][0]

    if tenant and tenant not in tenants:
        raise ValueError(f'Invalid tenant {tenant}')

    # compose result URL
    result = '{}/{}/{}/{}?size={}'.format(
        _EVE_IMAGE_SERVER_URL,
        endpoint,
        entity_id,
        variant,
        size
    )
    if tenant:
        result += f'&tenant={tenant}'

    return result


def alliance_logo_url(alliance_id: int, size: int = _DEFAULT_IMAGE_SIZE) -> str:
    """image URL for the given alliance ID"""
    return _eve_entity_image_url(_ESI_CATEGORY_ALLIANCE, alliance_id, size)


def corporation_logo_url(
    corporation_id: int, size: int = _DEFAULT_IMAGE_SIZE
) -> str:
    """image URL for the given corporation ID"""
    return _eve_entity_image_url(
        _ESI_CATEGORY_CORPORATION, corporation_id, size
    )


def character_portrait_url(
    character_id: int, size: int = _DEFAULT_IMAGE_SIZE
) -> str:
    """image URL for the given character ID"""
    return _eve_entity_image_url(_ESI_CATEGORY_CHARACTER, character_id, size)


def type_icon_url(type_id: int, size: int = _DEFAULT_IMAGE_SIZE) -> str:
    """icon image URL for the given type ID"""
    return _eve_entity_image_url(
        _ESI_CATEGORY_INVENTORYTYPE, type_id, size, variant='icon'
    )


def type_render_url(type_id: int, size: int = _DEFAULT_IMAGE_SIZE) -> str:
    """render image URL for the given type ID"""
    return _eve_entity_image_url(
        _ESI_CATEGORY_INVENTORYTYPE, type_id, size, variant='render'
    )



from typing import ClassVar
from django.db import models
from django.utils.translation import gettext_lazy as _

from allianceauth.menu.constants import DEFAULT_FOLDER_ICON_CLASSES

from .constants import DEFAULT_MENU_ITEM_ORDER, MenuItemType
from .core.menu_item_hooks import MenuItemHookCustom
from .managers import MenuItemManager


class MenuItem(models.Model):
    """An item in the sidebar menu.

    Some of these objects are generated from `MenuItemHook` objects.
    To avoid confusion we are using the same same field names.user defined
    """

    text = models.CharField(
        max_length=150,
        db_index=True,
        verbose_name=_("text"),
        help_text=_("Text to show on menu"),
    )
    order = models.IntegerField(
        default=DEFAULT_MENU_ITEM_ORDER,
        db_index=True,
        verbose_name=_("order"),
        help_text=_("Order of the menu. Lowest First"),
    )
    parent = models.ForeignKey(
        "self",
        on_delete=models.SET_NULL,
        default=None,
        null=True,
        blank=True,
        related_name="children",
        verbose_name=_("folder"),
        help_text=_("Folder this item is in (optional)"),
    )
    is_hidden = models.BooleanField(
        default=False,
        verbose_name=_("is hidden"),
        help_text=_(
            "Hide this menu item."
            "If this item is a folder all items under it will be hidden too"
        ),
    )

    # app related properties
    hook_hash = models.CharField(
        max_length=64, default=None, null=True, unique=True, editable=False
    )  # hash of a menu item hook. Must be nullable for unique comparison.

    # user defined properties
    classes = models.CharField(
        max_length=150,
        default="",
        blank=True,
        verbose_name=_("icon classes"),
        help_text=_(
            "Font Awesome classes to show as icon on menu, "
            "e.g. <code>fa-solid fa-house</code>"
        ),
    )
    url = models.TextField(
        default="",
        verbose_name=_("url"),
        help_text=_("External URL this menu items will link to"),
    )

    objects: ClassVar[MenuItemManager] = MenuItemManager()

    def __str__(self) -> str:
        return self.text

    def save(self, *args, **kwargs):
        if not self.hook_hash:
            self.hook_hash = None  # empty strings can create problems
        return super().save(*args, **kwargs)

    @property
    def item_type(self) -> MenuItemType:
        """Return the type of this menu item."""
        if self.hook_hash:
            return MenuItemType.APP

        if not self.url:
            return MenuItemType.FOLDER

        return MenuItemType.LINK

    @property
    def is_app_item(self) -> bool:
        """Return True if this is an app item, else False."""
        return self.item_type is MenuItemType.APP

    @property
    def is_child(self) -> bool:
        """Return True if this item is a child, else False."""
        return bool(self.parent_id)

    @property
    def is_folder(self) -> bool:
        """Return True if this item is a folder, else False."""
        return self.item_type is MenuItemType.FOLDER

    @property
    def is_link_item(self) -> bool:
        """Return True if this item is a link item, else False."""
        return self.item_type is MenuItemType.LINK

    @property
    def is_user_defined(self) -> bool:
        """Return True if this item is user defined."""
        return self.item_type is not MenuItemType.APP

    def to_hook_obj(self) -> MenuItemHookCustom:
        """Convert to hook object for rendering."""
        if self.is_app_item:
            raise ValueError("The related hook objects should be used for app items.")

        hook_obj = MenuItemHookCustom(
            text=self.text, classes=self.classes, url_name="", order=self.order
        )
        hook_obj.navactive = []
        if self.is_folder and not self.classes:
            hook_obj.classes = DEFAULT_FOLDER_ICON_CLASSES

        hook_obj.url = self.url
        hook_obj.is_folder = self.is_folder
        hook_obj.html_id = f"id-folder-{self.id}" if self.is_folder else ""
        return hook_obj



from unittest.mock import patch

from django.test import TestCase

from allianceauth.menu.core import smart_sync
from allianceauth.menu.tests.factories import create_link_menu_item
from allianceauth.menu.tests.utils import PACKAGE_PATH


@patch(PACKAGE_PATH + ".models.MenuItem.objects.sync_all", spec=True)
class TestSmartSync(TestCase):
    def test_should_sync_after_reset(self, mock_sync_all):
        # given
        smart_sync.reset_menu_items_sync()
        mock_sync_all.reset_mock()

        # when
        smart_sync.sync_menu()

        # then
        self.assertTrue(mock_sync_all.called)

    def test_should_sync_when_sync_flag_is_set_but_no_items_in_db(self, mock_sync_all):
        # given
        smart_sync._record_menu_was_synced()

        # when
        smart_sync.sync_menu()

        # then
        self.assertTrue(mock_sync_all.called)

    def test_should_not_sync_when_sync_flag_is_set_and_items_in_db(self, mock_sync_all):
        # given
        smart_sync._record_menu_was_synced()
        create_link_menu_item()

        # when
        smart_sync.sync_menu()

        # then
        self.assertFalse(mock_sync_all.called)



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class TimerBoardConfig(AppConfig):
    name = 'allianceauth.timerboard'
    label = 'timerboard'
    verbose_name = _('Structure Timers')



"""Template tags for rendering the classic side menu."""

from django import template
from django.http import HttpRequest

from allianceauth.hooks import get_hooks

register = template.Library()


# TODO: Show user created menu items
# TODO: Apply is_hidden feature for BS3 type items


@register.inclusion_tag("public/menublock.html", takes_context=True)
def menu_items(context: dict) -> dict:
    """Render menu items for classic dashboard."""
    items = render_menu(context["request"])
    return {"menu_items": items}


def render_menu(request: HttpRequest):
    """Return the rendered side menu for including in a template.

    This function is creating a BS3 style menu.
    """

    hooks = get_hooks("menu_item_hook")
    raw_items = [fn() for fn in hooks]
    raw_items.sort(key=lambda i: i.order)
    menu_items = [item.render(request) for item in raw_items]
    return menu_items



# Generated by Django 3.2.9 on 2021-11-25 18:38

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('groupmanagement', '0017_improve_groups_documentation'),
    ]

    operations = [
        migrations.CreateModel(
            name='ReservedGroupName',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='Name that can not be used for groups.', max_length=150, unique=True, verbose_name='name')),
                ('reason', models.TextField(help_text='Reason why this name is reserved.', verbose_name='reason')),
                ('created_by', models.CharField(help_text='Name of the user who created this entry.', max_length=255, verbose_name='created by')),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now, help_text='Date when this entry was created', verbose_name='created at')),
            ],
        ),
    ]



# Generated by Django 3.2.8 on 2021-10-19 18:53

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("groupmanagement", "0015_make_descriptions_great_again"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="grouprequest",
            name="status",
        ),
    ]



# Generated by Django 3.1.2 on 2020-10-25 11:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("groupmanagement", "0014_auto_20200918_1412"),
    ]

    operations = [
        migrations.AlterField(
            model_name="authgroup",
            name="description",
            field=models.TextField(
                blank=True,
                help_text="Short description <i>(max. 512 characters)</i> of the group shown to users.",
                max_length=512,
            ),
        ),
    ]



from django.test import RequestFactory, TestCase, override_settings
from django.urls import reverse

from allianceauth.groupmanagement.models import Group, GroupRequest
from allianceauth.tests.auth_utils import AuthUtils

from .. import views


def response_content_to_str(response) -> str:
    return response.content.decode(response.charset)


class TestViews(TestCase):

    def setUp(self):
        self.factory = RequestFactory()
        self.user = AuthUtils.create_user('Peter Parker')
        self.user_with_manage_permission = AuthUtils.create_user('Bruce Wayne')
        self.group = Group.objects.create(name="Example group")

        # set permissions
        AuthUtils.add_permission_to_user_by_name(
            'auth.group_management', self.user_with_manage_permission
        )

    def test_groups_view_can_load(self):
        request = self.factory.get(reverse('groupmanagement:groups'))
        request.user = self.user
        response = views.groups_view(request)
        self.assertEqual(response.status_code, 200)

    def test_management_view_can_load_for_user_with_permissions(self):
        """
        Test if user with management permissions can access the management view
        :return:
        """

        request = self.factory.get(reverse('groupmanagement:management'))
        request.user = self.user_with_manage_permission
        response = views.group_management(request)
        self.assertEqual(response.status_code, 200)

    def test_management_view_doesnt_load_for_user_without_permissions(self):
        """
        Test if user without management permissions can't access the management view
        :return:
        """

        request = self.factory.get(reverse('groupmanagement:management'))
        request.user = self.user
        response = views.group_management(request)
        self.assertEqual(response.status_code, 302)

    @override_settings(GROUPMANAGEMENT_AUTO_LEAVE=False)
    def test_leave_requests_tab_visible(self):
        """
        Test if the leave requests tab is visible when GROUPMANAGEMENT_AUTO_LEAVE = False
        :return:
        """

        request = self.factory.get(reverse('groupmanagement:management'))
        request.user = self.user_with_manage_permission
        response = views.group_management(request)

        content = response_content_to_str(response)

        self.assertEqual(response.status_code, 200)
        self.assertIn('id="leave-tab" data-bs-toggle="tab" data-bs-target="#leave"', content)
        self.assertIn('<div id="leave" class="tab-pane">', content)

    @override_settings(GROUPMANAGEMENT_AUTO_LEAVE=True)
    def test_leave_requests_tab_hidden(self):
        """
        Test if the leave requests tab is hidden when GROUPMANAGEMENT_AUTO_LEAVE = True
        :return:
        """

        request = self.factory.get(reverse('groupmanagement:management'))
        request.user = self.user_with_manage_permission
        response = views.group_management(request)

        content = response_content_to_str(response)

        self.assertEqual(response.status_code, 200)
        self.assertNotIn('id="leave-tab" data-bs-toggle="tab" data-bs-target="#leave"', content)
        self.assertNotIn('<div id="leave" class="tab-pane">', content)

    @override_settings(GROUPMANAGEMENT_AUTO_LEAVE=True)
    def test_should_not_hide_leave_requests_tab_when_there_are_open_requests(self):
        # given
        request = self.factory.get(reverse('groupmanagement:management'))
        request.user = self.user_with_manage_permission
        GroupRequest.objects.create(user=self.user, group=self.group, leave_request=True)

        # when
        response = views.group_management(request)

        # then
        content = response_content_to_str(response)
        self.assertEqual(response.status_code, 200)
        self.assertIn('<a class="nav-link" id="leave-tab" data-bs-toggle="tab" data-bs-target="#leave"', content)
        self.assertIn('<div id="leave" class="tab-pane">', content)



"""Admin site for menu app."""

from typing import Optional

from django.contrib import admin
from django.http import HttpRequest, HttpResponse
from django.shortcuts import get_object_or_404
from django.utils.translation import gettext_noop as _

from .constants import MenuItemType
from .core.smart_sync import sync_menu
from .filters import MenuItemTypeListFilter
from .forms import (
    AppMenuItemAdminForm,
    FolderMenuItemAdminForm,
    LinkMenuItemAdminForm,
)
from .models import MenuItem


@admin.register(MenuItem)
class MenuItemAdmin(admin.ModelAdmin):
    list_display = (
        "_text",
        "parent",
        "order",
        "_user_defined",
        "_visible",
        "_children",
    )
    list_filter = [
        MenuItemTypeListFilter,
        "is_hidden",
        ("parent", admin.RelatedOnlyFieldListFilter),
    ]
    ordering = ["parent", "order", "text"]

    def get_form(self, request: HttpRequest, obj: Optional[MenuItem] = None, **kwargs):
        kwargs["form"] = self._choose_form(request, obj)
        return super().get_form(request, obj, **kwargs)

    @classmethod
    def _choose_form(cls, request: HttpRequest, obj: Optional[MenuItem]):
        """Return the form for the current menu item type."""
        if obj:  # change
            if obj.hook_hash:
                return AppMenuItemAdminForm

            if obj.is_folder:
                return FolderMenuItemAdminForm

            return LinkMenuItemAdminForm

        # add
        if cls._type_from_request(request) is MenuItemType.FOLDER:
            return FolderMenuItemAdminForm

        return LinkMenuItemAdminForm

    def add_view(self, request, form_url="", extra_context=None) -> HttpResponse:
        context = extra_context or {}
        item_type = self._type_from_request(request, default=MenuItemType.LINK)
        context["title"] = _("Add %s menu item") % item_type.label
        return super().add_view(request, form_url, context)

    def change_view(
        self, request, object_id, form_url="", extra_context=None
    ) -> HttpResponse:
        extra_context = extra_context or {}
        obj = get_object_or_404(MenuItem, id=object_id)
        extra_context["title"] = _("Change %s menu item") % obj.item_type.label
        return super().change_view(request, object_id, form_url, extra_context)

    def changelist_view(self, request: HttpRequest, extra_context=None):
        # needed to ensure items are updated after an app change
        # and when the admin page is opened directly
        sync_menu()
        extra_context = extra_context or {}
        extra_context["folder_type"] = MenuItemType.FOLDER.value
        return super().changelist_view(request, extra_context)

    @admin.display(description=_("children"))
    def _children(self, obj: MenuItem):
        if not obj.is_folder:
            return []

        names = [obj.text for obj in obj.children.order_by("order", "text")]
        return names if names else "?"

    @admin.display(description=_("text"), ordering="text")
    def _text(self, obj: MenuItem) -> str:
        if obj.is_folder:
            return f"[{obj.text}]"
        return obj.text

    @admin.display(description=_("user defined"), boolean=True)
    def _user_defined(self, obj: MenuItem) -> bool:
        return obj.is_user_defined

    @admin.display(description=_("visible"), ordering="is_hidden", boolean=True)
    def _visible(self, obj: MenuItem) -> bool:
        return not bool(obj.is_hidden)

    @staticmethod
    def _type_from_request(
        request: HttpRequest, default=None
    ) -> Optional[MenuItemType]:
        try:
            return MenuItemType(request.GET.get("type"))
        except ValueError:
            return default



# Generated by Django 4.2.16 on 2025-01-20 06:16

import allianceauth.crontab.models
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CronOffset',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('minute', models.FloatField(default=allianceauth.crontab.models.random_default, verbose_name='Minute Offset')),
                ('hour', models.FloatField(default=allianceauth.crontab.models.random_default, verbose_name='Hour Offset')),
                ('day_of_month', models.FloatField(default=allianceauth.crontab.models.random_default, verbose_name='Day of Month Offset')),
                ('month_of_year', models.FloatField(default=allianceauth.crontab.models.random_default, verbose_name='Month of Year Offset')),
                ('day_of_week', models.FloatField(default=allianceauth.crontab.models.random_default, verbose_name='Day of Week Offset')),
            ],
            options={
                'verbose_name': 'Cron Offsets',
            },
        ),
    ]



# Generated by Django 1.10.1 on 2016-12-14 21:36

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('esi', '0002_scopes_20161208'),
        ('eveonline', '0004_eveapikeypair_sso_verified'),
    ]

    operations = [
        migrations.CreateModel(
            name='CorpStats',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('last_update', models.DateTimeField(auto_now=True)),
                ('_members', models.TextField(default='{}')),
                ('corp', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='eveonline.EveCorporationInfo')),
                ('token', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='esi.Token')),
            ],
            options={
                'default_permissions': ('add', 'change', 'remove', 'view_corp', 'view_alliance', 'view_blue'),
                'verbose_name': 'corp stats',
                'verbose_name_plural': 'corp stats',
                'permissions': (('corp_apis', 'Can view API keys of members of their corporation.'), ('alliance_apis', 'Can view API keys of members of their alliance.'), ('blue_apis', 'Can view API keys of members of blue corporations.')),
            },
        ),
    ]



from unittest import mock

from django.test import TestCase, RequestFactory
from django.contrib.auth.models import User, Group, Permission
from django.core.exceptions import ObjectDoesNotExist
from django.conf import settings

from allianceauth.tests.auth_utils import AuthUtils

from .auth_hooks import DiscourseService
from .models import DiscourseUser
from .tasks import DiscourseTasks

MODULE_PATH = 'allianceauth.services.modules.discourse'
DEFAULT_AUTH_GROUP = 'Member'


def add_permissions():
    permission = Permission.objects.get(codename='access_discourse')
    members = Group.objects.get_or_create(name=DEFAULT_AUTH_GROUP)[0]
    AuthUtils.add_permissions_to_groups([permission], [members])


class DiscourseHooksTestCase(TestCase):
    def setUp(self):
        self.member = 'member_user'
        member = AuthUtils.create_member(self.member)
        DiscourseUser.objects.create(user=member, enabled=True)
        self.none_user = 'none_user'
        none_user = AuthUtils.create_user(self.none_user)
        self.service = DiscourseService
        add_permissions()

    def test_has_account(self):
        member = User.objects.get(username=self.member)
        none_user = User.objects.get(username=self.none_user)
        self.assertTrue(DiscourseTasks.has_account(member))
        self.assertFalse(DiscourseTasks.has_account(none_user))

    def test_service_enabled(self):
        service = self.service()
        member = User.objects.get(username=self.member)
        none_user = User.objects.get(username=self.none_user)

        self.assertTrue(service.service_active_for_user(member))
        self.assertFalse(service.service_active_for_user(none_user))

    @mock.patch(MODULE_PATH + '.tasks.DiscourseManager')
    def test_update_all_groups(self, manager):
        service = self.service()
        service.update_all_groups()
        # Check member and blue user have groups updated
        self.assertTrue(manager.update_groups.called)
        self.assertEqual(manager.update_groups.call_count, 1)

    def test_update_groups(self):
        # Check member has Member group updated
        with mock.patch(MODULE_PATH + '.tasks.DiscourseManager') as manager:
            service = self.service()
            member = User.objects.get(username=self.member)
            service.update_groups(member)
            self.assertTrue(manager.update_groups.called)
            args, kwargs = manager.update_groups.call_args
            user, = args
            self.assertEqual(user, member)

        # Check none user does not have groups updated
        with mock.patch(MODULE_PATH + '.tasks.DiscourseManager') as manager:
            service = self.service()
            none_user = User.objects.get(username=self.none_user)
            service.update_groups(none_user)
            self.assertFalse(manager.update_groups.called)

    @mock.patch(MODULE_PATH + '.tasks.DiscourseManager')
    def test_validate_user(self, manager):
        service = self.service()
        # Test member is not deleted
        member = User.objects.get(username=self.member)
        service.validate_user(member)
        self.assertTrue(member.discourse)

        # Test none user is deleted
        none_user = User.objects.get(username=self.none_user)
        DiscourseUser.objects.create(user=none_user, enabled=True)
        service.validate_user(none_user)
        self.assertTrue(manager.disable_user.called)
        with self.assertRaises(ObjectDoesNotExist):
            none_discourse = User.objects.get(username=self.none_user).discourse

    @mock.patch(MODULE_PATH + '.tasks.DiscourseManager')
    def test_delete_user(self, manager):
        member = User.objects.get(username=self.member)

        service = self.service()
        result = service.delete_user(member)

        self.assertTrue(result)
        self.assertTrue(manager.disable_user.called)
        with self.assertRaises(ObjectDoesNotExist):
            discourse_user = User.objects.get(username=self.member).discourse

    def test_render_services_ctrl(self):
        service = self.service()
        member = User.objects.get(username=self.member)
        request = RequestFactory().get('/services/')
        request.user = member

        response = service.render_services_ctrl(request)
        self.assertTemplateUsed(service.service_ctrl_template)
        self.assertIn('href="%s"' % settings.DISCOURSE_URL, response)


class DiscourseViewsTestCase(TestCase):
    def setUp(self):
        self.member = AuthUtils.create_member('auth_member')
        AuthUtils.add_main_character(self.member, 'auth_member', '12345', corp_id='111', corp_name='Test Corporation')
        add_permissions()

    @mock.patch(MODULE_PATH + '.tasks.DiscourseManager')
    def test_sso_member(self, manager):
        self.client.force_login(self.member)
        data = {'sso': 'bm9uY2U9Y2I2ODI1MWVlZmI1MjExZTU4YzAwZmYxMzk1ZjBjMGI%3D%0A',
                'sig': '2828aa29899722b35a2f191d34ef9b3ce695e0e6eeec47deb46d588d70c7cb56'}
        response = self.client.get('/discourse/sso', data=data, follow=False)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url[:37], 'https://example.com/session/sso_login')



"""
Form widgets for custom_css app
"""

# Django
from django import forms

# Alliance Auth
from allianceauth.custom_css.models import CustomCSS


class CssEditorWidget(forms.Textarea):
    """
    Widget for editing CSS
    """

    def __init__(self, attrs=None):
        default_attrs = {"class": "custom-css-editor"}

        if attrs:
            default_attrs.update(attrs)

        super().__init__(default_attrs)

    # For when we want to add some sort of syntax highlight to it, which is not that
    # easy to do on a textarea field though.
    # `highlight.js` is just used as an example here, and doesn't work on a textarea field.
    # class Media:
    #     css = {
    #         "all": (
    #             "/static/custom_css/libs/highlight.js/11.10.0/styles/github.min.css",
    #         )
    #     }
    #     js = (
    #         "/static/custom_css/libs/highlight.js/11.10.0/highlight.min.js",
    #         "/static/custom_css/libs/highlight.js/11.10.0/languages/css.min.js",
    #         "/static/custom_css/javascript/custom-css.min.js",
    #     )



# Generated by Django 4.2.15 on 2024-08-14 11:25

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="CustomCSS",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "css",
                    models.TextField(
                        blank=True,
                        help_text="This CSS will be added to the site after the default CSS.",
                        null=True,
                        verbose_name="Your custom CSS",
                    ),
                ),
                ("timestamp", models.DateTimeField(auto_now=True)),
            ],
            options={
                "verbose_name": "Custom CSS",
                "verbose_name_plural": "Custom CSS",
                "default_permissions": (),
            },
        ),
    ]



from django.utils.translation import gettext_lazy as _

from allianceauth import hooks
from allianceauth.menu.hooks import MenuItemHook
from allianceauth.services.hooks import UrlHook

from . import urls
from .managers import SRPManager


class SrpMenu(MenuItemHook):
    def __init__(self):
        MenuItemHook.__init__(
            self, _('Ship Replacement'),
            'fa-regular fa-money-bill-1',
            'srp:management',
            navactive=['srp:']
        )

    def render(self, request):
        if request.user.has_perm('srp.access_srp'):
            app_count = SRPManager.pending_requests_count_for_user(request.user)
            self.count = app_count if app_count and app_count > 0 else None
            return MenuItemHook.render(self, request)
        return ''


@hooks.register('menu_item_hook')
def register_menu():
    return SrpMenu()


@hooks.register('url_hook')
def register_url():
    return UrlHook(urls, 'srp', r'^srp/')



import requests
import logging
from django.conf import settings
from django.apps import apps
from celery import shared_task
from .models import AnalyticsTokens, AnalyticsIdentifier
from .utils import (
    existence_baremetal_or_docker,
    install_stat_addons,
    install_stat_tokens,
    install_stat_users)

from allianceauth import __version__

logger = logging.getLogger(__name__)

BASE_URL = "https://www.google-analytics.com"

DEBUG_URL = f"{BASE_URL}/debug/mp/collect"
COLLECTION_URL = f"{BASE_URL}/mp/collect"

if getattr(settings, "ANALYTICS_ENABLE_DEBUG", False) and settings.DEBUG:
    # Force sending of analytics data during in a debug/test environment
    # Useful for developers working on this feature.
    logger.warning(
        "You have 'ANALYTICS_ENABLE_DEBUG' Enabled! "
        "This debug instance will send analytics data!")
    DEBUG_URL = COLLECTION_URL

ANALYTICS_URL = COLLECTION_URL

if settings.DEBUG is True:
    ANALYTICS_URL = DEBUG_URL


def analytics_event(namespace: str,
                    task: str,
                    label: str = "",
                    result: str = "",
                    value: int = 1,
                    event_type: str = 'Celery'):
    """
    Send a Google Analytics Event for each token stored
    Includes check for if its enabled/disabled

    Args:
        `namespace` (str): Celery Namespace
        `task` (str): Task Name
        `label` (str): Optional, additional task label
        `result` (str): Optional, Task Success/Exception
        `value` (int): Optional, If bulk, Query size, can be a Boolean
        `event_type` (str): Optional, Celery or Stats only, Default to Celery
    """
    for token in AnalyticsTokens.objects.filter(type='GA-V4'):
        if event_type == 'Stats':
            allowed = token.send_stats
        else:
            allowed = False

        if allowed is True:
            send_ga_tracking_celery_event.s(
                measurement_id=token.token,
                secret=token.secret,
                namespace=namespace,
                task=task,
                label=label,
                result=result,
                value=value).apply_async(priority=9)


@shared_task
def analytics_daily_stats() -> None:
    """Celery Task: Do not call directly

    Gathers a series of daily statistics
    Sends analytics events containing them
    """
    users = install_stat_users()
    tokens = install_stat_tokens()
    addons = install_stat_addons()
    existence_type = existence_baremetal_or_docker()
    logger.debug("Running Daily Analytics Upload")

    analytics_event(namespace='allianceauth.analytics',
                    task='send_install_stats',
                    label='existence',
                    value=1,
                    event_type='Stats')
    analytics_event(namespace='allianceauth.analytics',
                    task='send_install_stats',
                    label=existence_type,
                    value=1,
                    event_type='Stats')
    analytics_event(namespace='allianceauth.analytics',
                    task='send_install_stats',
                    label='users',
                    value=users,
                    event_type='Stats')
    analytics_event(namespace='allianceauth.analytics',
                    task='send_install_stats',
                    label='tokens',
                    value=tokens,
                    event_type='Stats')
    analytics_event(namespace='allianceauth.analytics',
                    task='send_install_stats',
                    label='addons',
                    value=addons,
                    event_type='Stats')
    for appconfig in apps.get_app_configs():
        if appconfig.label in [
            "django_celery_beat",
            "bootstrapform",
            "messages",
            "sessions",
            "auth",
            "staticfiles",
            "users",
            "addons",
            "admin",
            "humanize",
            "contenttypes",
            "sortedm2m",
            "django_bootstrap5",
            "tokens",
            "authentication",
            "services",
            "framework",
            "notifications"
            "eveonline",
            "navhelper",
            "analytics",
            "menu",
            "theme"
        ]:
            pass
        else:
            analytics_event(namespace='allianceauth.analytics',
                            task='send_extension_stats',
                            label=appconfig.label,
                            value=1,
                            event_type='Stats')


@shared_task
def send_ga_tracking_celery_event(
        measurement_id: str,
        secret: str,
        namespace: str,
        task: str,
        label: str = "",
        result: str = "",
        value: int = 1):
    """Celery Task: Do not call directly

    Sends an events to GA

    Parameters
    ----------
    `measurement_id` (str): GA Token
    `secret` (str): GA Authentication Secret
    `namespace` (str): Celery Namespace
    `task` (str): Task Name
    `label` (str): Optional, additional task label
    `result` (str): Optional, Task Success/Exception
    `value` (int): Optional, If bulk, Query size, can be a binary True/False
    """

    parameters = {
        'measurement_id': measurement_id,
        'api_secret': secret
    }

    payload = {
        'client_id': AnalyticsIdentifier.get_solo().identifier.hex,
        "user_properties": {
            "allianceauth_version": {
                "value": __version__
            }
        },
        'non_personalized_ads': True,
        "events": [{
            "name": "celery_event",
            "params": {
                    "namespace": namespace,
                    "task": task,
                    'result': result,
                    'label': label,
                    "value": value
            }
        }]
    }
    try:
        response = requests.post(
            ANALYTICS_URL,
            params=parameters,
            json=payload,
            timeout=10)
        response.raise_for_status()
        logger.debug(
            f"Analytics Celery/Stats Event HTTP{response.status_code}")
        return response.status_code
    except requests.exceptions.HTTPError as e:
        logger.debug(e)
        return response.status_code
    except requests.exceptions.ConnectionError as e:
        logger.debug(e)
        return "Failed"



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class Phpbb3ServiceConfig(AppConfig):
    name = 'allianceauth.services.modules.phpbb3'
    label = 'phpbb3'
    verbose_name = _('phpBB3 Service')



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class ServicesConfig(AppConfig):
    name = 'allianceauth.services'
    label = 'services'
    verbose_name = _('Services')

    def ready(self):
        from . import signals



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class CorpUtilsConfig(AppConfig):
    name = 'allianceauth.corputils'
    label = 'corputils'
    verbose_name = _('Corporation Stats')



# Generated by Django 2.0.6 on 2018-06-04 02:45

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('groupmanagement', '0008_remove_authgroup_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='RequestLog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('request_type', models.NullBooleanField(default=0)),
                ('request_info', models.CharField(max_length=254)),
                ('action', models.BooleanField(default=0)),
                ('group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='auth.Group')),
                ('request_actor', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]
